#if defined(MNH_IOCDF4)
MODULE MODE_NETCDF
USE MODD_NETCDF

IMPLICIT NONE 

PRIVATE

INTERFACE NCWRIT
   MODULE PROCEDURE NCWRITX0, NCWRITX1, NCWRITX2, NCWRITX3, &
        & NCWRITX4, NCWRITX5, NCWRITX6, &
        & NCWRITN0, NCWRITN1, NCWRITN2, &
        & NCWRITC0, NCWRITC1
END INTERFACE NCWRIT

INTERFACE NCREAD
   MODULE PROCEDURE NCREADX0, NCREADX1, NCREADX2, NCREADX3, &
        & NCREADX4, NCREADX5, NCREADX6, &
        & NCREADN0, NCREADN1, NCREADN2, &
        & NCREADC0
END INTERFACE NCREAD

! Public from netcdf.inc :
!PUBLIC NF_OPEN,NF_CREATE,NF_NOWRITE,NF_CLOBBER,NF_NETCDF4,NF_NOERR,NF_STRERROR
! Public from this module :
PUBLIC NEWIOCDF,CLEANIOCDF,NCWRIT,NCREAD

CONTAINS

FUNCTION NEWIOCDF()
TYPE(IOCDF), POINTER :: NEWIOCDF
TYPE(IOCDF), POINTER :: TZIOCDF
INTEGER              :: IRESP

ALLOCATE(TZIOCDF, STAT=IRESP)
IF (IRESP > 0) THEN 
  PRINT *, 'NEWIOCDF : memory allocation error...'
  STOP
END IF

TZIOCDF%NCID = -1
NULLIFY(TZIOCDF%DIMX)
NULLIFY(TZIOCDF%DIMY)
NULLIFY(TZIOCDF%DIMZ)
NULLIFY(TZIOCDF%DIMSTR)
NULLIFY(TZIOCDF%DIMLIST)

NEWIOCDF=>TZIOCDF

END FUNCTION NEWIOCDF

SUBROUTINE CLEANIOCDF(PIOCDF)
TYPE(IOCDF),  POINTER :: PIOCDF

INTEGER(KIND=IDCDF_KIND) :: IRESP

! Close Netcdf File
IRESP = NF_CLOSE(PIOCDF%NCID)
IF (IRESP /= NF_NOERR) THEN
   PRINT *, 'CLEANIOCDF, NF_CLOSE error : ', NF_STRERROR(IRESP)
END IF

! Clean DIMLIST and DIMSTR
CALL CLEANLIST(PIOCDF%DIMLIST)
CALL CLEANLIST(PIOCDF%DIMSTR)
! Then free iocdf
DEALLOCATE(PIOCDF)

PRINT *, 'CLEANIOCDF done.'

CONTAINS

SUBROUTINE CLEANLIST(PLIST)
TYPE(DIMCDF), POINTER :: PLIST,TZDIMCUR, TZDIMNEXT    

TZDIMCUR  => PLIST
DO WHILE(ASSOCIATED(TZDIMCUR))
   TZDIMNEXT => TZDIMCUR%NEXT
   DEALLOCATE(TZDIMCUR)
   TZDIMCUR => TZDIMNEXT
END DO

END SUBROUTINE CLEANLIST
  
END SUBROUTINE CLEANIOCDF

SUBROUTINE HANDLE_ERR(status,line,text,kresp)
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: status
INTEGER, INTENT(IN) :: line
CHARACTER(LEN=*), INTENT(IN) :: text
INTEGER, OPTIONAL, INTENT(OUT) :: kresp

! Don't stop the code when kresp is present
! and ensure kresp is a negative integer
IF (status /= NF_NOERR) THEN
   PRINT *, 'NETCDF ERROR in '//TRIM(text), line, NF_STRERROR(status)
   IF (PRESENT(kresp)) THEN
      IF (status < 0) THEN
         kresp = status
      ELSE IF (status == 0) THEN
         kresp = -1
      ELSE
         kresp = -status
      END IF
   ELSE
      STOP
   END IF
END IF
END SUBROUTINE HANDLE_ERR

FUNCTION str_replace(hstr, hold, hnew)
CHARACTER(LEN=*) :: hstr, hold, hnew
CHARACTER(LEN=LEN_TRIM(hstr)+MAX(0,LEN(hnew)-LEN(hold))) :: str_replace

INTEGER :: pos

pos = INDEX(hstr,hold)
IF (pos /= 0) THEN
   str_replace = hstr(1:pos-1)//hnew//hstr(pos+LEN(hold):)
ELSE 
   str_replace = hstr 
END IF

END FUNCTION str_replace

SUBROUTINE WRITATTR(KNCID, KVARID, TPFMH)
USE MODD_FM, ONLY : FMHEADER
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KNCID
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KVARID
TYPE(FMHEADER), INTENT(IN) :: TPFMH

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND),PARAMETER :: IONE = 1

! GRID attribute definition
STATUS = NF_PUT_ATT_INT(KNCID, KVARID, 'GRID', &
     &NF_INT, IONE, INT(TPFMH%GRID,KIND=IDCDF_KIND))
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'WRITATTR [NF_PUT_ATT_INT]')

! COMMENT attribute definition
STATUS = NF_PUT_ATT_TEXT(KNCID, KVARID,'COMMENT', &
     &INT(LEN_TRIM(TPFMH%COMMENT),KIND=IDCDF_KIND), TPFMH%COMMENT)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'WRITATTR [NF_PUT_ATT_TEXT]')

END SUBROUTINE WRITATTR

FUNCTION GETDIMCDF(PIOCDF, KLEN, HDIMNAME)
TYPE(IOCDF), POINTER       :: PIOCDF
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KLEN
CHARACTER(LEN=*), OPTIONAL :: HDIMNAME ! When provided don't search but
                                       ! simply create with name HDIMNAME
TYPE(DIMCDF), POINTER   :: GETDIMCDF

TYPE(DIMCDF), POINTER :: TMP
INTEGER               :: COUNT
CHARACTER(LEN=7)      :: YSUFFIX
CHARACTER(LEN=8)      :: YDIMNAME
INTEGER(KIND=IDCDF_KIND) :: STATUS

IF (KLEN < 1) THEN
   PRINT *, 'GETDIMCDF Error, KLEN=', KLEN
   STOP
END IF

IF (PRESENT(HDIMNAME)) THEN
   NULLIFY(TMP)
   YDIMNAME = TRIM(HDIMNAME)
ELSE
   ! Search dimension with KLEN length
   COUNT = 1
   TMP  => PIOCDF%DIMLIST
   DO WHILE(ASSOCIATED(TMP))
      IF (TMP%LEN == KLEN .AND. TMP%NAME /= 'STRLEN') EXIT
      TMP=>TMP%NEXT
      COUNT = COUNT+1
   END DO
   WRITE(YSUFFIX,'(i7)') KLEN
   YDIMNAME = 'D'//ADJUSTL(YSUFFIX)
END IF

IF (.NOT. ASSOCIATED(TMP)) THEN
   ! Not found then define new dimension
   ALLOCATE(TMP)
   TMP%NAME = YDIMNAME
   TMP%LEN = KLEN
   STATUS = NF_DEF_DIM(PIOCDF%NCID, TMP%NAME, KLEN, TMP%ID)
   IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'GETDIMCDF[NF_DEF_DIM]')
   NULLIFY(TMP%NEXT)
   TMP%NEXT       => PIOCDF%DIMLIST
   PIOCDF%DIMLIST => TMP
END IF

GETDIMCDF => TMP

END FUNCTION GETDIMCDF

FUNCTION GETSTRDIMID(PIOCDF, KLEN)
TYPE(IOCDF), POINTER    :: PIOCDF
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KLEN
INTEGER(KIND=IDCDF_KIND)            :: GETSTRDIMID

TYPE(DIMCDF), POINTER :: TMP
CHARACTER(LEN=7)      :: YSUFFIX
CHARACTER(LEN=8)      :: YDIMNAME
INTEGER(KIND=IDCDF_KIND) :: STATUS

IF (KLEN < 1) THEN
   PRINT *, 'GETSTRDIMID Error, KLEN=', KLEN
   STOP
END IF

! Search string dimension with KLEN length
TMP  => PIOCDF%DIMSTR
DO WHILE(ASSOCIATED(TMP))
   IF (TMP%LEN == KLEN) EXIT
   TMP=>TMP%NEXT
END DO
WRITE(YSUFFIX,'(i7)') KLEN
YDIMNAME = 'S'//ADJUSTL(YSUFFIX)

IF (.NOT. ASSOCIATED(TMP)) THEN
   ! Not found then define new dimension
   ALLOCATE(TMP)
   TMP%NAME = YDIMNAME
   TMP%LEN = KLEN
   STATUS = NF_DEF_DIM(PIOCDF%NCID, TMP%NAME, KLEN, TMP%ID)
   IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'GETSTRDIMID[NF_DEF_DIM]')
   NULLIFY(TMP%NEXT)
   TMP%NEXT      => PIOCDF%DIMSTR
   PIOCDF%DIMSTR => TMP
END IF

GETSTRDIMID = TMP%ID

END FUNCTION GETSTRDIMID

SUBROUTINE FILLVDIMS(PIOCDF, KSHAPE, HDIR, KVDIMS)
TYPE(IOCDF),           POINTER        :: PIOCDF 
INTEGER(KIND=IDCDF_KIND), DIMENSION(:), INTENT(IN) :: KSHAPE
CHARACTER(LEN=*),      INTENT(IN)     :: HDIR
INTEGER(KIND=IDCDF_KIND),DIMENSION(:), INTENT(OUT) :: KVDIMS

INTEGER :: II
TYPE(DIMCDF), POINTER :: PTDIM

IF (SIZE(KSHAPE) < 1) THEN
   PRINT *, 'FILLVDIMS Error, KSHAPE empty'
   STOP
END IF

DO II=1, SIZE(KSHAPE)

   IF (II == 1) THEN
      IF (HDIR == 'XX' .OR. HDIR == 'XY') THEN
         IF (.NOT. ASSOCIATED(PIOCDF%DIMX))  PIOCDF%DIMX => GETDIMCDF(PIOCDF, KSHAPE(II), 'X')
         IF (KSHAPE(II) == PIOCDF%DIMX%LEN) THEN
            PTDIM => PIOCDF%DIMX
         ELSE
            PTDIM => GETDIMCDF(PIOCDF, KSHAPE(II))
         END IF
      ELSE IF (HDIR == 'YY') THEN
         IF (.NOT. ASSOCIATED(PIOCDF%DIMY))  PIOCDF%DIMY => GETDIMCDF(PIOCDF, KSHAPE(II), 'Y')
         IF (KSHAPE(II) == PIOCDF%DIMY%LEN) THEN
            PTDIM => PIOCDF%DIMY
         ELSE
            PTDIM => GETDIMCDF(PIOCDF, KSHAPE(II))
         END IF
      ELSE
         PTDIM => GETDIMCDF(PIOCDF, KSHAPE(II))
         KVDIMS(II) = PTDIM%ID
      END IF
   ELSE IF (II == 2) THEN
      IF (HDIR == 'XY') THEN
         IF (.NOT. ASSOCIATED(PIOCDF%DIMY))  PIOCDF%DIMY => GETDIMCDF(PIOCDF, KSHAPE(II), 'Y')
         IF (KSHAPE(II) == PIOCDF%DIMY%LEN) THEN
            PTDIM => PIOCDF%DIMY
         ELSE
            PTDIM => GETDIMCDF(PIOCDF, KSHAPE(II))
         END IF
      ELSE
         PTDIM => GETDIMCDF(PIOCDF, KSHAPE(II))
      END IF
   ELSE IF (II == 3) THEN
      IF (HDIR == 'XY') THEN
         IF (.NOT. ASSOCIATED(PIOCDF%DIMZ))  PIOCDF%DIMZ => GETDIMCDF(PIOCDF, KSHAPE(II), 'Z')
         IF (KSHAPE(II) == PIOCDF%DIMZ%LEN) THEN
            PTDIM => PIOCDF%DIMZ
         ELSE
            PTDIM => GETDIMCDF(PIOCDF, KSHAPE(II))
         END IF
      ELSE
         PTDIM => GETDIMCDF(PIOCDF, KSHAPE(II))
      END IF
   ELSE
      PTDIM => GETDIMCDF(PIOCDF, KSHAPE(II))
   END IF
   
   KVDIMS(II) = PTDIM%ID
      
END DO

END SUBROUTINE FILLVDIMS


SUBROUTINE NCWRITX0(PZCDF, HVARNAME, HDIR, PFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER
TYPE(IOCDF), POINTER             :: PZCDF
CHARACTER(LEN=*),     INTENT(IN) :: HVARNAME
CHARACTER(LEN=*),     INTENT(IN) :: HDIR
REAL,                 INTENT(IN) :: PFIELD
TYPE(FMHEADER),       INTENT(IN) :: TPFMH
INTEGER,              INTENT(OUT):: KRESP

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND) :: INCID
CHARACTER(LEN=30) :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND),PARAMETER :: IZERO = 0
INTEGER           :: IRESP

IRESP = 0
! Get the Netcdf file ID
INCID = PZCDF%NCID

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! The variable should not already exist but who knows ?
STATUS = NF_INQ_VARID(INCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   ! Define the scalar variable 
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_DOUBLE, IZERO, IZERO, IVARID)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX0[NF_DEF_VAR]')
   CALL WRITATTR(INCID, IVARID, TPFMH)
ELSE
   PRINT *,'NCWRITX0 : ', TRIM(YVARNAME), ' already defined !'
END IF

! Write the data
STATUS = NF_PUT_VAR_DOUBLE(INCID, IVARID, PFIELD)
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX0[NF_PUT_VAR_DOUBLE]',IRESP)

KRESP = IRESP
END SUBROUTINE NCWRITX0

SUBROUTINE NCWRITX1(PZCDF, HVARNAME, HDIR, PFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER
TYPE(IOCDF), POINTER             :: PZCDF
CHARACTER(LEN=*),     INTENT(IN) :: HVARNAME
CHARACTER(LEN=*),     INTENT(IN) :: HDIR
REAL, DIMENSION(:),   INTENT(IN) :: PFIELD
TYPE(FMHEADER),       INTENT(IN) :: TPFMH
INTEGER,              INTENT(OUT):: KRESP

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND) :: INCID
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(PFIELD))) :: IVDIMS
INTEGER               :: IRESP

IRESP = 0
! Get the Netcdf file ID
INCID = PZCDF%NCID

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! The variable should not already exist but who knows ?
STATUS = NF_INQ_VARID(INCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   ! Get the netcdf dimensions
   CALL FILLVDIMS(PZCDF, INT(SHAPE(PFIELD),KIND=IDCDF_KIND), HDIR, IVDIMS)

   ! Define the variable 
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_DOUBLE, INT(SIZE(IVDIMS),KIND=IDCDF_KIND), IVDIMS, IVARID)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX1[NF_DEF_VAR]')
   CALL WRITATTR(INCID, IVARID, TPFMH)
ELSE
   PRINT *,'NCWRITX1 : ', TRIM(YVARNAME), ' already defined !'
END IF

! Write the data
STATUS = NF_PUT_VAR_DOUBLE(INCID, IVARID, PFIELD)
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX1[NF_PUT_VAR_DOUBLE]',IRESP)
 
KRESP = IRESP
END SUBROUTINE NCWRITX1

SUBROUTINE NCWRITX2(PZCDF, HVARNAME, HDIR, PFIELD, TPFMH, KRESP)
USE MODD_IO_ll, ONLY : LDEFLATEX2
USE MODD_FM, ONLY : FMHEADER
TYPE(IOCDF), POINTER             :: PZCDF
CHARACTER(LEN=*),     INTENT(IN) :: HVARNAME
CHARACTER(LEN=*),     INTENT(IN) :: HDIR
REAL, DIMENSION(:,:), INTENT(IN) :: PFIELD
TYPE(FMHEADER),       INTENT(IN) :: TPFMH
INTEGER,              INTENT(OUT):: KRESP

INTEGER(KIND=IDCDF_KIND),PARAMETER :: SHUFFLE = 0
INTEGER(KIND=IDCDF_KIND),PARAMETER :: DEFLATE = 1
INTEGER(KIND=IDCDF_KIND),PARAMETER :: DEFLATE_LEVEL = 2

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND) :: INCID
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(PFIELD))) :: IVDIMS
INTEGER               :: IRESP

IRESP = 0
! Get the Netcdf file ID
INCID = PZCDF%NCID

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! The variable should not already exist but who knows ?
STATUS = NF_INQ_VARID(INCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   ! Get the netcdf dimensions
   CALL FILLVDIMS(PZCDF, INT(SHAPE(PFIELD),KIND=IDCDF_KIND), HDIR, IVDIMS)
   
   ! Define the variable 
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_DOUBLE, INT(SIZE(IVDIMS),KIND=IDCDF_KIND), IVDIMS, IVARID)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX2[NF_DEF_VAR]')
   IF (LDEFLATEX2) THEN
      ! Compress the variable with deflate level 2
      STATUS = NF_DEF_VAR_DEFLATE(INCID, IVARID, SHUFFLE, DEFLATE, DEFLATE_LEVEL)
      IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX2[NF_DEF_VAR_DEFLATE]')
   END IF
   CALL WRITATTR(INCID, IVARID, TPFMH)
ELSE
   PRINT *,'NCWRITX2 : ', TRIM(YVARNAME), ' already defined !'
END IF

! Write the data
STATUS = NF_PUT_VAR_DOUBLE(INCID, IVARID, PFIELD)
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX2[NF_PUT_VAR_DOUBLE]',IRESP)
 
KRESP = IRESP
END SUBROUTINE NCWRITX2

SUBROUTINE NCWRITX3(PZCDF, HVARNAME, HDIR, PFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER
TYPE(IOCDF), POINTER              :: PZCDF
CHARACTER(LEN=*),      INTENT(IN) :: HVARNAME
CHARACTER(LEN=*),      INTENT(IN) :: HDIR
REAL, DIMENSION(:,:,:),INTENT(IN) :: PFIELD
TYPE(FMHEADER),        INTENT(IN) :: TPFMH
INTEGER,               INTENT(OUT):: KRESP

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND) :: INCID
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(PFIELD))) :: IVDIMS
INTEGER               :: IRESP

IRESP = 0
! Get the Netcdf file ID
INCID = PZCDF%NCID

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! The variable should not already exist but who knows ?
STATUS = NF_INQ_VARID(INCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   ! Get the netcdf dimensions
   CALL FILLVDIMS(PZCDF, INT(SHAPE(PFIELD),KIND=IDCDF_KIND), HDIR, IVDIMS)

   ! Define the variable 
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_DOUBLE, INT(SIZE(IVDIMS),KIND=IDCDF_KIND), IVDIMS, IVARID)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX3[NF_DEF_VAR]')
   CALL WRITATTR(INCID, IVARID, TPFMH)
ELSE
   PRINT *,'NCWRITX3 : ', TRIM(YVARNAME), ' already defined !'
END IF

! Write the data
STATUS = NF_PUT_VAR_DOUBLE(INCID, IVARID, PFIELD)
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX3[NF_PUT_VAR_DOUBLE] '//TRIM(HVARNAME),IRESP)
 
KRESP = IRESP
END SUBROUTINE NCWRITX3

SUBROUTINE NCWRITX4(PZCDF, HVARNAME, HDIR, PFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER
TYPE(IOCDF), POINTER                 :: PZCDF
CHARACTER(LEN=*),         INTENT(IN) :: HVARNAME
CHARACTER(LEN=*),         INTENT(IN) :: HDIR
REAL, DIMENSION(:,:,:,:), INTENT(IN) :: PFIELD
TYPE(FMHEADER),           INTENT(IN) :: TPFMH
INTEGER,                  INTENT(OUT):: KRESP

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND) :: INCID
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(PFIELD))) :: IVDIMS
INTEGER               :: IRESP

IRESP = 0
! Get the Netcdf file ID
INCID = PZCDF%NCID

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! The variable should not already exist but who knows ?
STATUS = NF_INQ_VARID(INCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   ! Get the netcdf dimensions
   CALL FILLVDIMS(PZCDF, INT(SHAPE(PFIELD),KIND=IDCDF_KIND), HDIR, IVDIMS)

   ! Define the variable 
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_DOUBLE, INT(SIZE(IVDIMS),KIND=IDCDF_KIND), IVDIMS, IVARID)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX4[NF_DEF_VAR]')
   CALL WRITATTR(INCID, IVARID, TPFMH)
ELSE
   PRINT *,'NCWRITX4 : ', TRIM(YVARNAME), ' already defined !'
END IF

! Write the data
STATUS = NF_PUT_VAR_DOUBLE(INCID, IVARID, PFIELD)
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX4[NF_PUT_VAR_DOUBLE]',IRESP)
 
KRESP = IRESP
END SUBROUTINE NCWRITX4

SUBROUTINE NCWRITX5(PZCDF, HVARNAME, HDIR, PFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER
TYPE(IOCDF), POINTER                   :: PZCDF
CHARACTER(LEN=*),           INTENT(IN) :: HVARNAME
CHARACTER(LEN=*),           INTENT(IN) :: HDIR
REAL, DIMENSION(:,:,:,:,:), INTENT(IN) :: PFIELD
TYPE(FMHEADER),             INTENT(IN) :: TPFMH
INTEGER,                    INTENT(OUT):: KRESP

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND) :: INCID
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(PFIELD))) :: IVDIMS
INTEGER               :: IRESP

IRESP = 0
! Get the Netcdf file ID
INCID = PZCDF%NCID

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! The variable should not already exist but who knows ?
STATUS = NF_INQ_VARID(INCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   ! Get the netcdf dimensions
   CALL FILLVDIMS(PZCDF, INT(SHAPE(PFIELD),KIND=IDCDF_KIND), HDIR, IVDIMS)

   ! Define the variable 
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_DOUBLE, INT(SIZE(IVDIMS),KIND=IDCDF_KIND), IVDIMS, IVARID)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX5[NF_DEF_VAR]')
   CALL WRITATTR(INCID, IVARID, TPFMH)
ELSE
   PRINT *,'NCWRITX5 : ', TRIM(YVARNAME), ' already defined !'
END IF

! Write the data
STATUS = NF_PUT_VAR_DOUBLE(INCID, IVARID, PFIELD)
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX5[NF_PUT_VAR_DOUBLE]',IRESP)
 
KRESP = IRESP
END SUBROUTINE NCWRITX5

SUBROUTINE NCWRITX6(PZCDF, HVARNAME, HDIR, PFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER
TYPE(IOCDF), POINTER                     :: PZCDF
CHARACTER(LEN=*),             INTENT(IN) :: HVARNAME
CHARACTER(LEN=*),             INTENT(IN) :: HDIR
REAL, DIMENSION(:,:,:,:,:,:), INTENT(IN) :: PFIELD
TYPE(FMHEADER),               INTENT(IN) :: TPFMH
INTEGER,                      INTENT(OUT):: KRESP

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND) :: INCID
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(PFIELD))) :: IVDIMS
INTEGER               :: IRESP

IRESP = 0
! Get the Netcdf file ID
INCID = PZCDF%NCID

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! The variable should not already exist but who knows ?
STATUS = NF_INQ_VARID(INCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   ! Get the netcdf dimensions
   CALL FILLVDIMS(PZCDF, INT(SHAPE(PFIELD),KIND=IDCDF_KIND), HDIR, IVDIMS)
   
   ! Define the variable 
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_DOUBLE, INT(SIZE(IVDIMS),KIND=IDCDF_KIND), IVDIMS, IVARID)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX6[NF_DEF_VAR]')
   CALL WRITATTR(INCID, IVARID, TPFMH)
ELSE
   PRINT *,'NCWRITX6 : ', TRIM(YVARNAME), ' already defined !'
END IF

! Write the data
STATUS = NF_PUT_VAR_DOUBLE(INCID, IVARID, PFIELD)
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITX6[NF_PUT_VAR_DOUBLE]',IRESP)
 
KRESP = IRESP
END SUBROUTINE NCWRITX6

SUBROUTINE NCWRITN0(PZCDF, HVARNAME, HDIR, KFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER
USE MODD_PARAMETERS_ll,  ONLY : JPHEXT, JPVEXT
USE MODD_IO_ll, ONLY : LPACK,L1D,L2D
TYPE(IOCDF), POINTER             :: PZCDF
CHARACTER(LEN=*),     INTENT(IN) :: HVARNAME
CHARACTER(LEN=*),     INTENT(IN) :: HDIR
INTEGER,              INTENT(IN) :: KFIELD
TYPE(FMHEADER),       INTENT(IN) :: TPFMH
INTEGER,              INTENT(OUT):: KRESP

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND) :: INCID
CHARACTER(LEN=30) :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND),PARAMETER :: IZERO = 0
INTEGER           :: IRESP

IRESP = 0
! Get the Netcdf file ID
INCID = PZCDF%NCID

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! The variable should not already exist but who knows ?
STATUS = NF_INQ_VARID(INCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   ! Define the scalar variable 
#ifndef MNH_INT8
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_INT, IZERO, IZERO, IVARID)
#else
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_INT64, IZERO, IZERO, IVARID)
#endif
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITN0[NF_DEF_VAR]')
   CALL WRITATTR(INCID, IVARID, TPFMH)
ELSE
   PRINT *,'NCWRITN0 : ', TRIM(YVARNAME), ' already defined !'
END IF

! Write the data
#ifndef MNH_INT8
STATUS = NF_PUT_VAR_INT(INCID, IVARID, INT(KFIELD,KIND=IDCDF_KIND))
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITN0[NF_PUT_VAR_INT]',IRESP)
#else
STATUS = NF_PUT_VAR_INT64(INCID, IVARID, KFIELD)
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITN0[NF_PUT_VAR_INT64]',IRESP)
#endif
!
! Use IMAX, JMAX, KMAX to define DIMX, DIMY, DIMZ
! /!\ Can only work if IMAX, JMAX or KMAX are written before any array
!
#if 0
IF (YVARNAME == 'IMAX' .AND. .NOT. ASSOCIATED(PZCDF%DIMX)) PZCDF%DIMX=>GETDIMCDF(PZCDF,KFIELD+2*JPHEXT,'X')
IF (YVARNAME == 'JMAX' .AND. .NOT. ASSOCIATED(PZCDF%DIMY)) THEN
   IF (LPACK .AND. L2D) THEN
      PZCDF%DIMY=>GETDIMCDF(PZCDF, 1,'Y')
   ELSE
      PZCDF%DIMY=>GETDIMCDF(PZCDF, KFIELD+2*JPHEXT, 'Y')
   END IF
END IF
#endif
IF (YVARNAME == 'KMAX' .AND. .NOT. ASSOCIATED(PZCDF%DIMZ)) PZCDF%DIMZ=>GETDIMCDF(PZCDF,INT(KFIELD+2*JPVEXT,KIND=IDCDF_KIND),'Z')
 
KRESP = IRESP
END SUBROUTINE NCWRITN0

SUBROUTINE NCWRITN1(PZCDF, HVARNAME, HDIR, KFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER
TYPE(IOCDF), POINTER              :: PZCDF
CHARACTER(LEN=*),      INTENT(IN) :: HVARNAME
CHARACTER(LEN=*),      INTENT(IN) :: HDIR
INTEGER, DIMENSION(:), INTENT(IN) :: KFIELD
TYPE(FMHEADER),        INTENT(IN) :: TPFMH
INTEGER,               INTENT(OUT):: KRESP

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND) :: INCID
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(KFIELD))) :: IVDIMS
INTEGER               :: IRESP

IRESP = 0
! Get the Netcdf file ID
INCID = PZCDF%NCID

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! The variable should not already exist but who knows ?
STATUS = NF_INQ_VARID(INCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   ! Get the netcdf dimensions
   CALL FILLVDIMS(PZCDF, INT(SHAPE(KFIELD),KIND=IDCDF_KIND), HDIR, IVDIMS)
! Define the variable 
#ifndef MNH_INT8
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_INT, INT(SIZE(IVDIMS),KIND=IDCDF_KIND), IVDIMS, IVARID)
#else
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_INT64, INT(SIZE(IVDIMS),KIND=IDCDF_KIND), IVDIMS, IVARID)
#endif
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITN1[NF_DEF_VAR] '//TRIM(YVARNAME))
   CALL WRITATTR(INCID, IVARID, TPFMH)
ELSE
   PRINT *,'NCWRITN1 : ', TRIM(YVARNAME), ' already defined !'
END IF

! Write the data
#ifndef MNH_INT8
STATUS = NF_PUT_VAR_INT(INCID, IVARID, INT(KFIELD,KIND=IDCDF_KIND))
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITN1[NF_PUT_VAR_INT]',IRESP)
#else
STATUS = NF_PUT_VAR_INT64(INCID, IVARID, KFIELD)
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITN1[NF_PUT_VAR_INT64]',IRESP)
#endif

KRESP = IRESP 
END SUBROUTINE NCWRITN1

SUBROUTINE NCWRITN2(PZCDF, HVARNAME, HDIR, KFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER
TYPE(IOCDF), POINTER               :: PZCDF
CHARACTER(LEN=*),       INTENT(IN) :: HVARNAME
CHARACTER(LEN=*),       INTENT(IN) :: HDIR
INTEGER, DIMENSION(:,:),INTENT(IN) :: KFIELD
TYPE(FMHEADER),         INTENT(IN) :: TPFMH
INTEGER,                INTENT(OUT):: KRESP

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND) :: INCID
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(KFIELD))) :: IVDIMS
INTEGER               :: IRESP

IRESP = 0
! Get the Netcdf file ID
INCID = PZCDF%NCID

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! The variable should not already exist but who knows ?
STATUS = NF_INQ_VARID(INCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   ! Get the netcdf dimensions
   CALL FILLVDIMS(PZCDF, INT(SHAPE(KFIELD),KIND=IDCDF_KIND), HDIR, IVDIMS)

   ! Define the variable 
#ifndef MNH_INT8
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_INT, INT(SIZE(IVDIMS),KIND=IDCDF_KIND), IVDIMS, IVARID)
#else
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_INT64, INT(SIZE(IVDIMS),KIND=IDCDF_KIND), IVDIMS, IVARID)
#endif
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITN2[NF_DEF_VAR]')
   CALL WRITATTR(INCID, IVARID, TPFMH)
ELSE
   PRINT *,'NCWRITN2 : ', TRIM(YVARNAME), ' already defined !'
END IF

! Write the data
#ifndef MNH_INT8
STATUS = NF_PUT_VAR_INT(INCID, IVARID, INT(KFIELD,KIND=IDCDF_KIND))
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITN2[NF_PUT_VAR_INT]',IRESP)
#else
STATUS = NF_PUT_VAR_INT64(INCID, IVARID, KFIELD)
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITN2[NF_PUT_VAR_INT64]',IRESP)
#endif

KRESP = IRESP
END SUBROUTINE NCWRITN2

SUBROUTINE NCWRITC0(PZCDF, HVARNAME, HDIR, HFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER
TYPE(IOCDF), POINTER              :: PZCDF
CHARACTER(LEN=*),      INTENT(IN) :: HVARNAME
CHARACTER(LEN=*),      INTENT(IN) :: HDIR
CHARACTER(LEN=*),      INTENT(IN) :: HFIELD
TYPE(FMHEADER),        INTENT(IN) :: TPFMH
INTEGER,               INTENT(OUT):: KRESP

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND) :: INCID
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND), DIMENSION(1) :: IVDIMS
CHARACTER(LEN=32)     :: YSTR
!CHARACTER(LEN=LEN(HFIELD)) :: YSTR
INTEGER               :: IRESP

IRESP = 0
YSTR = HFIELD
IF (LEN_TRIM(HFIELD) > LEN(YSTR)) THEN
   PRINT *,'NCWRIT0 : ',TRIM(YVARNAME), ' string variable TRUNCATED.'
END IF

! Get the Netcdf file ID
INCID = PZCDF%NCID

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! The variable should not already exist but who knows ?
STATUS = NF_INQ_VARID(INCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   ! Get the netcdf string dimensions id 
   IVDIMS(1) = GETSTRDIMID(PZCDF, INT(LEN(YSTR),KIND=IDCDF_KIND))
   ! Define the variable 
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_CHAR, INT(SIZE(IVDIMS),KIND=IDCDF_KIND), IVDIMS, IVARID)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITC0[NF_DEF_VAR]')
   CALL WRITATTR(INCID, IVARID, TPFMH)
ELSE
   PRINT *,'NCWRITC0 : ', TRIM(YVARNAME), ' already defined !'
END IF

! Write the data
STATUS = NF_PUT_VAR_TEXT(INCID, IVARID, YSTR)
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITC0[NF_PUT_VAR_TEXT]',IRESP)
 
KRESP = IRESP
END SUBROUTINE NCWRITC0

SUBROUTINE NCWRITC1(PZCDF, HVARNAME, HDIR, HFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER
TYPE(IOCDF),        POINTER              :: PZCDF
CHARACTER(LEN=*),             INTENT(IN) :: HVARNAME
CHARACTER(LEN=*),             INTENT(IN) :: HDIR
CHARACTER(LEN=*),DIMENSION(:),INTENT(IN) :: HFIELD
TYPE(FMHEADER),               INTENT(IN) :: TPFMH
INTEGER,                      INTENT(OUT):: KRESP

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND) :: INCID
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND), DIMENSION(2) :: IVDIMS
INTEGER(KIND=IDCDF_KIND), DIMENSION(1) :: ITMP
INTEGER               :: IRESP
INTEGER(KIND=IDCDF_KIND) :: ILEN
INTEGER(KIND=IDCDF_KIND) :: ISIZE
INTEGER(KIND=IDCDF_KIND),PARAMETER :: IONE=1

IRESP = 0
ILEN  = LEN(HFIELD)
ISIZE = SIZE(HFIELD)

! Get the Netcdf file ID
INCID = PZCDF%NCID

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! The variable should not already exist but who knows ?
STATUS = NF_INQ_VARID(INCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   ! Get the netcdf dimensions ID 
   IVDIMS(1) = GETSTRDIMID(PZCDF,ILEN)
   CALL FILLVDIMS(PZCDF, (/ISIZE/), HDIR, ITMP)
   IVDIMS(2) = ITMP(1)
   ! Define the variable 
   STATUS = NF_DEF_VAR(INCID, YVARNAME, NF_CHAR, INT(SIZE(IVDIMS),KIND=IDCDF_KIND), IVDIMS, IVARID)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITC1[NF_DEF_VAR]')
   CALL WRITATTR(INCID, IVARID, TPFMH)
ELSE
   PRINT *,'NCWRITC1 : ', TRIM(YVARNAME), ' already defined !'
END IF

! Write the data
STATUS = NF_PUT_VARA_TEXT(INCID, IVARID, (/IONE,IONE/),(/ILEN,ISIZE/), HFIELD)
IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCWRITC1[NF_PUT_VARA_TEXT]',IRESP)
 
KRESP = IRESP
END SUBROUTINE NCWRITC1

!
!
! Here come the NetCDF READ routines
!
!
SUBROUTINE READATTR(KNCID, KVARID, HVAR, TPFMH)
USE MODD_FM, ONLY : FMHEADER, JPXKRK
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KNCID
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KVARID
CHARACTER(LEN=*),INTENT(IN) :: HVAR
TYPE(FMHEADER),  INTENT(OUT):: TPFMH

INTEGER(KIND=IDCDF_KIND) :: STATUS
INTEGER(KIND=IDCDF_KIND) :: ICOMLEN
      
! Read variables attributes (GRID and COMMENT)
STATUS = NF_GET_ATT_INT(KNCID, KVARID, 'GRID', TPFMH%GRID)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'READATTR[NF_GET_ATT_INT] '//TRIM(HVAR))
STATUS = NF_INQ_ATTLEN(KNCID, KVARID, 'COMMENT', ICOMLEN)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'READATTR[NF_INQ_ATTLEN] '//TRIM(HVAR))
IF (ICOMLEN <= JPXKRK) THEN
   TPFMH%COMLEN = ICOMLEN
   STATUS = NF_GET_ATT_TEXT(KNCID, KVARID, 'COMMENT', TPFMH%COMMENT)
   IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'READATTR[NF_GET_ATT_TEXT] '//TRIM(HVAR))
ELSE
   PRINT *, 'READATTR : '//TRIM(HVAR)//' COMMENT attribute ignored because too long.'
   TPFMH%COMLEN = 0
END IF
END SUBROUTINE READATTR

SUBROUTINE NCREADX0(KNCID, HVARNAME, PFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER, JPXKRK
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KNCID
CHARACTER(LEN=*), INTENT(IN) :: HVARNAME
REAL,             INTENT(OUT):: PFIELD
TYPE(FMHEADER),   INTENT(OUT):: TPFMH
INTEGER,          INTENT(OUT):: KRESP  ! return-code

INTEGER(KIND=IDCDF_KIND) :: STATUS
CHARACTER(LEN=30) :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND) :: ITYPE   ! variable type
INTEGER(KIND=IDCDF_KIND) :: IDIMS   ! number of dimensions
INTEGER(KIND=IDCDF_KIND) :: ICOMLEN ! comment length
INTEGER           :: IRESP

IRESP = 0

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! Get variable ID, NDIMS and TYPE
STATUS = NF_INQ_VARID(KNCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   CALL HANDLE_ERR(status,__LINE__,'NCREADX0[NF_INQ_VARID] '//TRIM(YVARNAME),IRESP)
   GOTO 1000
END IF
STATUS = NF_INQ_VARNDIMS(KNCID, IVARID, IDIMS)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX0[NF_INQ_VARNDIMS] '//TRIM(YVARNAME))
STATUS = NF_INQ_VARTYPE(KNCID, IVARID, ITYPE)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX0[NF_INQ_VARTYPE] '//TRIM(YVARNAME))

IF (IDIMS == 0 .AND. ITYPE == NF_DOUBLE) THEN
   ! Read variable
   STATUS = NF_GET_VAR_DOUBLE(KNCID, IVARID, PFIELD)
   IF (STATUS /= NF_NOERR) THEN
      CALL HANDLE_ERR(status,__LINE__,'NCREADX0[NF_GET_VAR_DOUBLE] '//TRIM(YVARNAME),IRESP)
      GOTO 1000
   END IF
   ! Read variables attributes (GRID and COMMENT)
   CALL READATTR(KNCID, IVARID, YVARNAME, TPFMH)
ELSE
   PRINT *, 'NCREADNCREADX0 : '//TRIM(YVARNAME)//' not READ (wrong size or type).'
   IRESP = -3
END IF

1000 CONTINUE
KRESP = IRESP

END SUBROUTINE NCREADX0

SUBROUTINE NCREADX1(KNCID, HVARNAME, PFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER, JPXKRK
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KNCID
CHARACTER(LEN=*),   INTENT(IN) :: HVARNAME
REAL, DIMENSION(:), INTENT(OUT):: PFIELD
TYPE(FMHEADER),     INTENT(OUT):: TPFMH
INTEGER,            INTENT(OUT):: KRESP  ! return-code

INTEGER(KIND=IDCDF_KIND) :: STATUS
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND) :: ITYPE   ! variable type
INTEGER(KIND=IDCDF_KIND) :: IDIMS   ! number of dimensions
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(PFIELD))) :: IVDIMS
INTEGER(KIND=IDCDF_KIND) :: ICOMLEN ! comment length
INTEGER               :: IVARSIZE
INTEGER(KIND=IDCDF_KIND) :: IDIMLEN
INTEGER               :: II
INTEGER               :: IRESP

IRESP = 0

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! Get variable ID, NDIMS and TYPE
STATUS = NF_INQ_VARID(KNCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   CALL HANDLE_ERR(status,__LINE__,'NCREADX1[NF_INQ_VARID] '//TRIM(YVARNAME),IRESP)
   GOTO 1000
END IF
STATUS = NF_INQ_VARNDIMS(KNCID, IVARID, IDIMS)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX1[NF_INQ_VARNDIMS] '//TRIM(YVARNAME))
STATUS = NF_INQ_VARTYPE(KNCID, IVARID, ITYPE)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX1[NF_INQ_VARTYPE] '//TRIM(YVARNAME))

IF (IDIMS == SIZE(SHAPE(PFIELD)) .AND. ITYPE == NF_DOUBLE) THEN
   ! Check size of variable before reading
   STATUS = NF_INQ_VARDIMID(KNCID, IVARID, IVDIMS)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADX1[NF_INQ_VARDIMID] '//TRIM(YVARNAME))
   IVARSIZE = 1
   DO II=1,IDIMS
      STATUS = NF_INQ_DIMLEN(KNCID,IVDIMS(II),IDIMLEN)
      IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADX1[NF_INQ_DIMLEN] '//TRIM(YVARNAME))
      IVARSIZE = IVARSIZE*IDIMLEN
   END DO
   
   IF (IVARSIZE == SIZE(PFIELD)) THEN
      ! Read variable
      STATUS = NF_GET_VAR_DOUBLE(KNCID, IVARID, PFIELD)
      IF (STATUS /= NF_NOERR) THEN
         CALL HANDLE_ERR(status,__LINE__,'NCREADX1[NF_GET_VAR_DOUBLE] '//TRIM(YVARNAME),IRESP)
         GOTO 1000
      END IF
      ! Read variables attributes (GRID and COMMENT)
      CALL READATTR(KNCID, IVARID, YVARNAME, TPFMH)
   ELSE
      PRINT *, 'NCREADX1 : '//TRIM(YVARNAME)//' not READ wrong size (file, mem) : ', IVARSIZE, SIZE(PFIELD)
      IRESP = -3
   END IF
ELSE
   PRINT *, 'NCREADX1 : '//TRIM(YVARNAME)//' not READ (wrong shape or type).'
   IRESP = -3
END IF

1000 CONTINUE
KRESP = IRESP

END SUBROUTINE NCREADX1

SUBROUTINE NCREADX2(KNCID, HVARNAME, PFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER, JPXKRK
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KNCID
CHARACTER(LEN=*),     INTENT(IN) :: HVARNAME
REAL, DIMENSION(:,:), INTENT(OUT):: PFIELD
TYPE(FMHEADER),       INTENT(OUT):: TPFMH
INTEGER,              INTENT(OUT):: KRESP  ! return-code

INTEGER(KIND=IDCDF_KIND) :: STATUS
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND) :: ITYPE   ! variable type
INTEGER(KIND=IDCDF_KIND) :: IDIMS   ! number of dimensions
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(PFIELD))) :: IVDIMS
INTEGER(KIND=IDCDF_KIND) :: ICOMLEN ! comment length
INTEGER               :: IVARSIZE
INTEGER(KIND=IDCDF_KIND) :: IDIMLEN
INTEGER               :: II
INTEGER               :: IRESP

IRESP = 0

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! Get variable ID, NDIMS and TYPE
STATUS = NF_INQ_VARID(KNCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   CALL HANDLE_ERR(status,__LINE__,'NCREADX2[NF_INQ_VARID] '//TRIM(YVARNAME),IRESP)
   GOTO 1000
END IF
STATUS = NF_INQ_VARNDIMS(KNCID, IVARID, IDIMS)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX2[NF_INQ_VARNDIMS] '//TRIM(YVARNAME))
STATUS = NF_INQ_VARTYPE(KNCID, IVARID, ITYPE)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX2[NF_INQ_VARTYPE] '//TRIM(YVARNAME))

IF (IDIMS == SIZE(SHAPE(PFIELD)) .AND. ITYPE == NF_DOUBLE) THEN
   ! Check size of variable before reading
   STATUS = NF_INQ_VARDIMID(KNCID, IVARID, IVDIMS)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADX2[NF_INQ_VARDIMID] '//TRIM(YVARNAME))
   IVARSIZE = 1
   DO II=1,IDIMS
      STATUS = NF_INQ_DIMLEN(KNCID,IVDIMS(II),IDIMLEN)
      IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADX2[NF_INQ_DIMLEN] '//TRIM(YVARNAME))
      IVARSIZE = IVARSIZE*IDIMLEN
   END DO
   
   IF (IVARSIZE == SIZE(PFIELD)) THEN
      ! Read variable
      STATUS = NF_GET_VAR_DOUBLE(KNCID, IVARID, PFIELD)
      IF (STATUS /= NF_NOERR) THEN
         CALL HANDLE_ERR(status,__LINE__,'NCREADX2[NF_GET_VAR_DOUBLE] '//TRIM(YVARNAME),IRESP)
         GOTO 1000
      END IF
      ! Read variables attributes (GRID and COMMENT)
      CALL READATTR(KNCID, IVARID, YVARNAME, TPFMH)
   ELSE
      PRINT *, 'NCREADX2 : '//TRIM(YVARNAME)//' not READ (wrong size).'
      IRESP = -3
   END IF
ELSE
   PRINT *, 'NCREADX2 : '//TRIM(YVARNAME)//' not READ (wrong shape or type).'
   IRESP = -3
END IF

1000 CONTINUE
KRESP = IRESP

END SUBROUTINE NCREADX2

SUBROUTINE NCREADX3(KNCID, HVARNAME, PFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER, JPXKRK
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KNCID
CHARACTER(LEN=*),       INTENT(IN) :: HVARNAME
REAL, DIMENSION(:,:,:), INTENT(OUT):: PFIELD
TYPE(FMHEADER),         INTENT(OUT):: TPFMH
INTEGER,                INTENT(OUT):: KRESP  ! return-code

INTEGER(KIND=IDCDF_KIND) :: STATUS
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND) :: ITYPE   ! variable type
INTEGER(KIND=IDCDF_KIND) :: IDIMS   ! number of dimensions
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(PFIELD))) :: IVDIMS
INTEGER(KIND=IDCDF_KIND) :: ICOMLEN ! comment length
INTEGER               :: IVARSIZE
INTEGER(KIND=IDCDF_KIND) :: IDIMLEN
INTEGER               :: II
INTEGER               :: IRESP

IRESP = 0

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! Get variable ID, NDIMS and TYPE
STATUS = NF_INQ_VARID(KNCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   CALL HANDLE_ERR(status,__LINE__,'NCREADX3[NF_INQ_VARID] '//TRIM(YVARNAME),IRESP)
   GOTO 1000
END IF
STATUS = NF_INQ_VARNDIMS(KNCID, IVARID, IDIMS)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX3[NF_INQ_VARNDIMS] '//TRIM(YVARNAME))
STATUS = NF_INQ_VARTYPE(KNCID, IVARID, ITYPE)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX3[NF_INQ_VARTYPE] '//TRIM(YVARNAME))

IF (IDIMS == SIZE(SHAPE(PFIELD)) .AND. ITYPE == NF_DOUBLE) THEN
   ! Check size of variable before reading
   STATUS = NF_INQ_VARDIMID(KNCID, IVARID, IVDIMS)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADX3[NF_INQ_VARDIMID] '//TRIM(YVARNAME))
   IVARSIZE = 1
   DO II=1,IDIMS
      STATUS = NF_INQ_DIMLEN(KNCID,IVDIMS(II),IDIMLEN)
      IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADX3[NF_INQ_DIMLEN] '//TRIM(YVARNAME))
      IVARSIZE = IVARSIZE*IDIMLEN
   END DO
   
   IF (IVARSIZE == SIZE(PFIELD)) THEN
      ! Read variable
      STATUS = NF_GET_VAR_DOUBLE(KNCID, IVARID, PFIELD)
      IF (STATUS /= NF_NOERR) THEN
         CALL HANDLE_ERR(status,__LINE__,'NCREADX3[NF_GET_VAR_DOUBLE] '//TRIM(YVARNAME),IRESP)
         GOTO 1000
      END IF
      ! Read variables attributes (GRID and COMMENT)
      CALL READATTR(KNCID, IVARID, YVARNAME, TPFMH)
   ELSE
      PRINT *, 'NCREADX3 : '//TRIM(YVARNAME)//' not READ (wrong size).'
      IRESP = -3
   END IF
ELSE
   PRINT *, 'NCREADX3 : '//TRIM(YVARNAME)//' not READ (wrong shape or type).'
   IRESP = -3
END IF

1000 CONTINUE
KRESP = IRESP

END SUBROUTINE NCREADX3

SUBROUTINE NCREADX4(KNCID, HVARNAME, PFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER, JPXKRK
INTEGER(KIND=IDCDF_KIND), INTENT(IN) :: KNCID
CHARACTER(LEN=*),         INTENT(IN) :: HVARNAME
REAL, DIMENSION(:,:,:,:), INTENT(OUT):: PFIELD
TYPE(FMHEADER),           INTENT(OUT):: TPFMH
INTEGER,                  INTENT(OUT):: KRESP  ! return-code

INTEGER(KIND=IDCDF_KIND) :: STATUS
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND) :: ITYPE   ! variable type
INTEGER(KIND=IDCDF_KIND) :: IDIMS   ! number of dimensions
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(PFIELD))) :: IVDIMS
INTEGER(KIND=IDCDF_KIND) :: ICOMLEN ! comment length
INTEGER               :: IVARSIZE
INTEGER(KIND=IDCDF_KIND) :: IDIMLEN
INTEGER               :: II
INTEGER               :: IRESP

IRESP = 0

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! Get variable ID, NDIMS and TYPE
STATUS = NF_INQ_VARID(KNCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   CALL HANDLE_ERR(status,__LINE__,'NCREADX4[NF_INQ_VARID] '//TRIM(YVARNAME),IRESP)
   GOTO 1000
END IF
STATUS = NF_INQ_VARNDIMS(KNCID, IVARID, IDIMS)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX4[NF_INQ_VARNDIMS] '//TRIM(YVARNAME))
STATUS = NF_INQ_VARTYPE(KNCID, IVARID, ITYPE)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX4[NF_INQ_VARTYPE] '//TRIM(YVARNAME))

IF (IDIMS == SIZE(SHAPE(PFIELD)) .AND. ITYPE == NF_DOUBLE) THEN
   ! Check size of variable before reading
   STATUS = NF_INQ_VARDIMID(KNCID, IVARID, IVDIMS)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADX4[NF_INQ_VARDIMID] '//TRIM(YVARNAME))
   IVARSIZE = 1
   DO II=1,IDIMS
      STATUS = NF_INQ_DIMLEN(KNCID,IVDIMS(II),IDIMLEN)
      IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADX4[NF_INQ_DIMLEN] '//TRIM(YVARNAME))
      IVARSIZE = IVARSIZE*IDIMLEN
   END DO
   
   IF (IVARSIZE == SIZE(PFIELD)) THEN
      ! Read variable
      STATUS = NF_GET_VAR_DOUBLE(KNCID, IVARID, PFIELD)
      IF (STATUS /= NF_NOERR) THEN
         CALL HANDLE_ERR(status,__LINE__,'NCREADX4[NF_GET_VAR_DOUBLE] '//TRIM(YVARNAME),IRESP)
         GOTO 1000
      END IF
      ! Read variables attributes (GRID and COMMENT)
      CALL READATTR(KNCID, IVARID, YVARNAME, TPFMH)
   ELSE
      PRINT *, 'NCREADX4 : '//TRIM(YVARNAME)//' not READ (wrong size).'
      IRESP = -3
   END IF
ELSE
   PRINT *, 'NCREADX4 : '//TRIM(YVARNAME)//' not READ (wrong shape or type).'
   IRESP = -3
END IF

1000 CONTINUE
KRESP = IRESP

END SUBROUTINE NCREADX4

SUBROUTINE NCREADX5(KNCID, HVARNAME, PFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER, JPXKRK
INTEGER(KIND=IDCDF_KIND),   INTENT(IN) :: KNCID
CHARACTER(LEN=*),           INTENT(IN) :: HVARNAME
REAL, DIMENSION(:,:,:,:,:), INTENT(OUT):: PFIELD
TYPE(FMHEADER),             INTENT(OUT):: TPFMH
INTEGER,                    INTENT(OUT):: KRESP  ! return-code

INTEGER(KIND=IDCDF_KIND) :: STATUS
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND) :: ITYPE   ! variable type
INTEGER(KIND=IDCDF_KIND) :: IDIMS   ! number of dimensions
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(PFIELD))) :: IVDIMS
INTEGER(KIND=IDCDF_KIND) :: ICOMLEN ! comment length
INTEGER               :: IVARSIZE
INTEGER(KIND=IDCDF_KIND) :: IDIMLEN
INTEGER               :: II
INTEGER               :: IRESP

IRESP = 0

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! Get variable ID, NDIMS and TYPE
STATUS = NF_INQ_VARID(KNCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   CALL HANDLE_ERR(status,__LINE__,'NCREADX5[NF_INQ_VARID] '//TRIM(YVARNAME),IRESP)
   GOTO 1000
END IF
STATUS = NF_INQ_VARNDIMS(KNCID, IVARID, IDIMS)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX5[NF_INQ_VARNDIMS] '//TRIM(YVARNAME))
STATUS = NF_INQ_VARTYPE(KNCID, IVARID, ITYPE)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX5[NF_INQ_VARTYPE] '//TRIM(YVARNAME))

IF (IDIMS == SIZE(SHAPE(PFIELD)) .AND. ITYPE == NF_DOUBLE) THEN
   ! Check size of variable before reading
   STATUS = NF_INQ_VARDIMID(KNCID, IVARID, IVDIMS)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADX5[NF_INQ_VARDIMID] '//TRIM(YVARNAME))
   IVARSIZE = 1
   DO II=1,IDIMS
      STATUS = NF_INQ_DIMLEN(KNCID,IVDIMS(II),IDIMLEN)
      IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADX5[NF_INQ_DIMLEN] '//TRIM(YVARNAME))
      IVARSIZE = IVARSIZE*IDIMLEN
   END DO
   
   IF (IVARSIZE == SIZE(PFIELD)) THEN
      ! Read variable
      STATUS = NF_GET_VAR_DOUBLE(KNCID, IVARID, PFIELD)
      IF (STATUS /= NF_NOERR) THEN
         CALL HANDLE_ERR(status,__LINE__,'NCREADX5[NF_GET_VAR_DOUBLE] '//TRIM(YVARNAME),IRESP)
         GOTO 1000
      END IF
      ! Read variables attributes (GRID and COMMENT)
      CALL READATTR(KNCID, IVARID, YVARNAME, TPFMH)
   ELSE
      PRINT *, 'NCREADX5 : '//TRIM(YVARNAME)//' not READ (wrong size).'
      IRESP = -3
   END IF
ELSE
   PRINT *, 'NCREADX5 : '//TRIM(YVARNAME)//' not READ (wrong shape or type).'
   IRESP = -3
END IF

1000 CONTINUE
KRESP = IRESP

END SUBROUTINE NCREADX5

SUBROUTINE NCREADX6(KNCID, HVARNAME, PFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER, JPXKRK
INTEGER(KIND=IDCDF_KIND),     INTENT(IN) :: KNCID
CHARACTER(LEN=*),             INTENT(IN) :: HVARNAME
REAL, DIMENSION(:,:,:,:,:,:), INTENT(OUT):: PFIELD
TYPE(FMHEADER),               INTENT(OUT):: TPFMH
INTEGER,                      INTENT(OUT):: KRESP  ! return-code

INTEGER(KIND=IDCDF_KIND) :: STATUS
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND) :: ITYPE   ! variable type
INTEGER(KIND=IDCDF_KIND) :: IDIMS   ! number of dimensions
INTEGER(KIND=IDCDF_KIND), DIMENSION(SIZE(SHAPE(PFIELD))) :: IVDIMS
INTEGER(KIND=IDCDF_KIND) :: ICOMLEN ! comment length
INTEGER               :: IVARSIZE
INTEGER(KIND=IDCDF_KIND) :: IDIMLEN
INTEGER               :: II
INTEGER               :: IRESP

IRESP = 0

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! Get variable ID, NDIMS and TYPE
STATUS = NF_INQ_VARID(KNCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   CALL HANDLE_ERR(status,__LINE__,'NCREADX6[NF_INQ_VARID] '//TRIM(YVARNAME),IRESP)
   GOTO 1000
END IF
STATUS = NF_INQ_VARNDIMS(KNCID, IVARID, IDIMS)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX6[NF_INQ_VARNDIMS] '//TRIM(YVARNAME))
STATUS = NF_INQ_VARTYPE(KNCID, IVARID, ITYPE)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADX6[NF_INQ_VARTYPE] '//TRIM(YVARNAME))

IF (IDIMS == SIZE(SHAPE(PFIELD)) .AND. ITYPE == NF_DOUBLE) THEN
   ! Check size of variable before reading
   STATUS = NF_INQ_VARDIMID(KNCID, IVARID, IVDIMS)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADX6[NF_INQ_VARDIMID] '//TRIM(YVARNAME))
   IVARSIZE = 1
   DO II=1,IDIMS
      STATUS = NF_INQ_DIMLEN(KNCID,IVDIMS(II),IDIMLEN)
      IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADX6[NF_INQ_DIMLEN] '//TRIM(YVARNAME))
      IVARSIZE = IVARSIZE*IDIMLEN
   END DO
   
   IF (IVARSIZE == SIZE(PFIELD)) THEN
      ! Read variable
      STATUS = NF_GET_VAR_DOUBLE(KNCID, IVARID, PFIELD)
      IF (STATUS /= NF_NOERR) THEN
         CALL HANDLE_ERR(status,__LINE__,'NCREADX6[NF_GET_VAR_DOUBLE] '//TRIM(YVARNAME),IRESP)
         GOTO 1000
      END IF
      ! Read variables attributes (GRID and COMMENT)
      CALL READATTR(KNCID, IVARID, YVARNAME, TPFMH)
   ELSE
      PRINT *, 'NCREADX6 : '//TRIM(YVARNAME)//' not READ (wrong size).'
      IRESP = -3
   END IF
ELSE
   PRINT *, 'NCREADX6 : '//TRIM(YVARNAME)//' not READ (wrong shape or type).'
   IRESP = -3
END IF

1000 CONTINUE
KRESP = IRESP

END SUBROUTINE NCREADX6

SUBROUTINE NCREADN0(KNCID, HVARNAME, KFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER, JPXKRK
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KNCID
CHARACTER(LEN=*), INTENT(IN) :: HVARNAME
INTEGER,          INTENT(OUT):: KFIELD
TYPE(FMHEADER),   INTENT(OUT):: TPFMH
INTEGER,          INTENT(OUT):: KRESP  ! return-code

INTEGER(KIND=IDCDF_KIND) :: STATUS
CHARACTER(LEN=30) :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND) :: ITYPE   ! variable type
INTEGER(KIND=IDCDF_KIND) :: IDIMS   ! number of dimensions
INTEGER(KIND=IDCDF_KIND) :: ICOMLEN ! comment length
INTEGER           :: IRESP

IRESP = 0

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! Get variable ID, NDIMS and TYPE
STATUS = NF_INQ_VARID(KNCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   CALL HANDLE_ERR(status,__LINE__,'NCREADN0[NF_INQ_VARID] '//TRIM(YVARNAME),IRESP)
   GOTO 1000
END IF
STATUS = NF_INQ_VARNDIMS(KNCID, IVARID, IDIMS)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADN0[NF_INQ_VARNDIMS] '//TRIM(YVARNAME))
STATUS = NF_INQ_VARTYPE(KNCID, IVARID, ITYPE)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADN0[NF_INQ_VARTYPE] '//TRIM(YVARNAME))

#ifndef MNH_INT8
IF (IDIMS == 0 .AND. ITYPE == NF_INT) THEN
#else
IF (IDIMS == 0 .AND. ITYPE == NF_INT64) THEN
#endif
! Read variable
#ifndef MNH_INT8
   STATUS = NF_GET_VAR_INT(KNCID, IVARID, KFIELD)
#else
   STATUS = NF_GET_VAR_INT64(KNCID, IVARID, KFIELD)
#endif
   IF (STATUS /= NF_NOERR) THEN
      CALL HANDLE_ERR(status,__LINE__,'NCREADN0[NF_GET_VAR_INT] '//TRIM(YVARNAME),IRESP)
      GOTO 1000
   END IF
   ! Read variables attributes (GRID and COMMENT)
   CALL READATTR(KNCID, IVARID, YVARNAME, TPFMH)
ELSE
   PRINT *, 'NCREADN0 : '//TRIM(YVARNAME)//' not READ (wrong size or type).'
   IRESP = -3
END IF

1000 CONTINUE
KRESP = IRESP

END SUBROUTINE NCREADN0

SUBROUTINE NCREADN1(KNCID, HVARNAME, KFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER, JPXKRK
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KNCID
CHARACTER(LEN=*),      INTENT(IN) :: HVARNAME
INTEGER, DIMENSION(:), INTENT(OUT):: KFIELD
TYPE(FMHEADER),        INTENT(OUT):: TPFMH
INTEGER,               INTENT(OUT):: KRESP  ! return-code

INTEGER(KIND=IDCDF_KIND) :: STATUS
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND) :: ITYPE   ! variable type
INTEGER(KIND=IDCDF_KIND) :: IDIMS   ! number of dimensions
INTEGER(KIND=IDCDF_KIND),DIMENSION(SIZE(SHAPE(KFIELD))) :: IVDIMS
INTEGER(KIND=IDCDF_KIND) :: ICOMLEN ! comment length
INTEGER               :: IVARSIZE
INTEGER(KIND=IDCDF_KIND) :: IDIMLEN
INTEGER               :: II
INTEGER               :: IRESP

IRESP = 0

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! Get variable ID, NDIMS and TYPE
STATUS = NF_INQ_VARID(KNCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   CALL HANDLE_ERR(status,__LINE__,'NCREADN1[NF_INQ_VARID] '//TRIM(YVARNAME),IRESP)
   GOTO 1000
END IF
STATUS = NF_INQ_VARNDIMS(KNCID, IVARID, IDIMS)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADN1[NF_INQ_VARNDIMS] '//TRIM(YVARNAME))
STATUS = NF_INQ_VARTYPE(KNCID, IVARID, ITYPE)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADN1[NF_INQ_VARTYPE] '//TRIM(YVARNAME))

#ifndef MNH_INT8
IF (IDIMS == SIZE(SHAPE(KFIELD)) .AND. ITYPE == NF_INT) THEN
#else
IF (IDIMS == SIZE(SHAPE(KFIELD)) .AND. ITYPE == NF_INT64) THEN
#endif
   ! Check size of variable before reading
   STATUS = NF_INQ_VARDIMID(KNCID, IVARID, IVDIMS)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADN1[NF_INQ_VARDIMID] '//TRIM(YVARNAME))
   IVARSIZE = 1
   DO II=1,IDIMS
      STATUS = NF_INQ_DIMLEN(KNCID,IVDIMS(II),IDIMLEN)
      IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADN1[NF_INQ_DIMLEN] '//TRIM(YVARNAME))
      IVARSIZE = IVARSIZE*IDIMLEN
   END DO
   
   IF (IVARSIZE == SIZE(KFIELD)) THEN
      ! Read variable
#ifndef MNH_INT8
      STATUS = NF_GET_VAR_INT(KNCID, IVARID, KFIELD)
#else
      STATUS = NF_GET_VAR_INT64(KNCID, IVARID, KFIELD)
#endif
      IF (STATUS /= NF_NOERR) THEN
         CALL HANDLE_ERR(status,__LINE__,'NCREADN1[NF_GET_VAR_INT] '//TRIM(YVARNAME),IRESP)
         GOTO 1000
      END IF
      ! Read variables attributes (GRID and COMMENT)
      CALL READATTR(KNCID, IVARID, YVARNAME, TPFMH)
   ELSE
      PRINT *, 'NCREADN1 : '//TRIM(YVARNAME)//' not READ (wrong size).'
      IRESP = -3
   END IF
ELSE
   PRINT *, 'NCREADN1 : '//TRIM(YVARNAME)//' not READ (wrong shape or type).'
   IRESP = -3
END IF

1000 CONTINUE
KRESP = IRESP

END SUBROUTINE NCREADN1

SUBROUTINE NCREADN2(KNCID, HVARNAME, KFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER, JPXKRK
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KNCID
CHARACTER(LEN=*),        INTENT(IN) :: HVARNAME
INTEGER, DIMENSION(:,:), INTENT(OUT):: KFIELD
TYPE(FMHEADER),          INTENT(OUT):: TPFMH
INTEGER,                 INTENT(OUT):: KRESP  ! return-code

INTEGER(KIND=IDCDF_KIND)  :: STATUS
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND) :: ITYPE   ! variable type
INTEGER(KIND=IDCDF_KIND) :: IDIMS   ! number of dimensions
INTEGER(KIND=IDCDF_KIND),DIMENSION(SIZE(SHAPE(KFIELD))) :: IVDIMS
INTEGER(KIND=IDCDF_KIND) :: ICOMLEN ! comment length
INTEGER               :: IVARSIZE
INTEGER(KIND=IDCDF_KIND) :: IDIMLEN
INTEGER               :: II
INTEGER               :: IRESP

IRESP = 0

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! Get variable ID, NDIMS and TYPE
STATUS = NF_INQ_VARID(KNCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   CALL HANDLE_ERR(status,__LINE__,'NCREADN2[NF_INQ_VARID] '//TRIM(YVARNAME),IRESP)
   GOTO 1000
END IF
STATUS = NF_INQ_VARNDIMS(KNCID, IVARID, IDIMS)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADN2[NF_INQ_VARNDIMS] '//TRIM(YVARNAME))
STATUS = NF_INQ_VARTYPE(KNCID, IVARID, ITYPE)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADN2[NF_INQ_VARTYPE] '//TRIM(YVARNAME))

#ifndef MNH_INT8
IF (IDIMS == SIZE(SHAPE(KFIELD)) .AND. ITYPE == NF_INT) THEN
#else
IF (IDIMS == SIZE(SHAPE(KFIELD)) .AND. ITYPE == NF_INT64) THEN
#endif
! Check size of variable before reading
   STATUS = NF_INQ_VARDIMID(KNCID, IVARID, IVDIMS)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADN2[NF_INQ_VARDIMID] '//TRIM(YVARNAME))
   IVARSIZE = 1
   DO II=1,IDIMS
      STATUS = NF_INQ_DIMLEN(KNCID,IVDIMS(II),IDIMLEN)
      IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADN2[NF_INQ_DIMLEN] '//TRIM(YVARNAME))
      IVARSIZE = IVARSIZE*IDIMLEN
   END DO
   
   IF (IVARSIZE == SIZE(KFIELD)) THEN
      ! Read variable
#ifndef MNH_INT8
      STATUS = NF_GET_VAR_INT(KNCID, IVARID, KFIELD)
#else
      STATUS = NF_GET_VAR_INT64(KNCID, IVARID, KFIELD)
#endif
      IF (STATUS /= NF_NOERR) THEN
         CALL HANDLE_ERR(status,__LINE__,'NCREADN2[NF_GET_VAR_INT] '//TRIM(YVARNAME),IRESP)
         GOTO 1000
      END IF
      ! Read variables attributes (GRID and COMMENT)
      CALL READATTR(KNCID, IVARID, YVARNAME, TPFMH)
   ELSE
      PRINT *, 'NCREADN2 : '//TRIM(YVARNAME)//' not READ (wrong size).'
      IRESP = -3
   END IF
ELSE
   PRINT *, 'NCREADN2 : '//TRIM(YVARNAME)//' not READ (wrong shape or type).'
   IRESP = -3
END IF

1000 CONTINUE
KRESP = IRESP

END SUBROUTINE NCREADN2

SUBROUTINE NCREADC0(KNCID, HVARNAME, HFIELD, TPFMH, KRESP)
USE MODD_FM, ONLY : FMHEADER, JPXKRK
INTEGER(KIND=IDCDF_KIND),INTENT(IN) :: KNCID
CHARACTER(LEN=*),      INTENT(IN) :: HVARNAME
CHARACTER(LEN=*),      INTENT(OUT):: HFIELD
TYPE(FMHEADER),        INTENT(OUT):: TPFMH
INTEGER,               INTENT(OUT):: KRESP  ! return-code

INTEGER(KIND=IDCDF_KIND) :: STATUS
CHARACTER(LEN=30)     :: YVARNAME
INTEGER(KIND=IDCDF_KIND) :: IVARID
INTEGER(KIND=IDCDF_KIND) :: ITYPE   ! variable type
INTEGER(KIND=IDCDF_KIND) :: IDIMS   ! number of dimensions
INTEGER(KIND=IDCDF_KIND),DIMENSION(1) :: IVDIMS
CHARACTER(LEN=32)     :: YSTR
!CHARACTER(LEN=LEN(HFIELD))     :: YSTR
INTEGER(KIND=IDCDF_KIND) :: ICOMLEN ! comment length
INTEGER(KIND=IDCDF_KIND) :: IDIMLEN
INTEGER               :: II
INTEGER               :: IRESP

IRESP = 0

! NetCDF var names can't contain '%' nor '.' 
YVARNAME = str_replace(HVARNAME, '%', '__')
YVARNAME = str_replace(YVARNAME, '.', '--')

! Get variable ID, NDIMS and TYPE
STATUS = NF_INQ_VARID(KNCID, YVARNAME, IVARID)
IF (STATUS /= NF_NOERR) THEN
   CALL HANDLE_ERR(status,__LINE__,'NCREADC0[NF_INQ_VARID] '//TRIM(YVARNAME),IRESP)
   GOTO 1000
END IF
STATUS = NF_INQ_VARNDIMS(KNCID, IVARID, IDIMS)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADC0[NF_INQ_VARNDIMS] '//TRIM(YVARNAME))
STATUS = NF_INQ_VARTYPE(KNCID, IVARID, ITYPE)
IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(STATUS,__LINE__,'NCREADC0[NF_INQ_VARTYPE] '//TRIM(YVARNAME))

IF (IDIMS == 1 .AND. ITYPE == NF_CHAR) THEN
   ! Check size of variable before reading
   STATUS = NF_INQ_VARDIMID(KNCID, IVARID, IVDIMS)
   IF (status /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADC0[NF_INQ_VARDIMID] '//TRIM(YVARNAME))
   STATUS = NF_INQ_DIMLEN(KNCID,IVDIMS(1),IDIMLEN)
   IF (STATUS /= NF_NOERR) CALL HANDLE_ERR(status,__LINE__,'NCREADC0[NF_INQ_DIMLEN] '//TRIM(YVARNAME))
   
   IF (IDIMLEN <= LEN(YSTR)) THEN
      ! Read variable
      STATUS = NF_GET_VAR_TEXT(KNCID, IVARID, YSTR)
      IF (STATUS /= NF_NOERR) THEN
         CALL HANDLE_ERR(status,__LINE__,'NCREADC0[NF_GET_VAR_TEXT] '//TRIM(YVARNAME),IRESP)
         GOTO 1000
      END IF
      IF (LEN_TRIM(YSTR) > LEN(HFIELD)) PRINT *, 'NCDREADC0 : '//TRIM(YVARNAME)//' truncated !!'
      HFIELD = TRIM(YSTR)
      ! Read variables attributes (GRID and COMMENT)
      CALL READATTR(KNCID, IVARID, YVARNAME, TPFMH)
   ELSE
      PRINT *, 'NCREADC0 : '//TRIM(YVARNAME)//' not READ (wrong size).'
      IRESP = -3
   END IF
ELSE
   PRINT *, 'NCREADC0 : '//TRIM(YVARNAME)//' not READ (wrong shape or type).'
   IRESP = -3
END IF

1000 CONTINUE
KRESP = IRESP

END SUBROUTINE NCREADC0

END MODULE MODE_NETCDF

#else
!
! External dummy subroutines
!
SUBROUTINE NCWRIT(A,B,C,D,E,F)
INTEGER :: A,B,C,D,E,F
PRINT *, 'NCWRIT empty call. Compile with -DMNH_IOCDF4 flag to enable NetCDF4 I/Os.'
END SUBROUTINE NCWRIT

SUBROUTINE NCREAD(A,B,C,D,E)
INTEGER :: A,B,C,D,E
PRINT *, 'NCREAD empty call. Compile with -DMNH_IOCDF4 flag to enable NetCDF4 I/Os.'
END SUBROUTINE NCREAD

SUBROUTINE CLEANIOCDF(A)
INTEGER :: A
PRINT *, 'CLEANIOCDF empty call. Compile with -DMNH_IOCDF4 flag to enable NetCDF4 I/Os.'
END SUBROUTINE CLEANIOCDF

#endif
