!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ################
      MODULE MODD_BEM_n
!     ################
!
!!****  *MODD_BEM_n - declaration of parameters and option for BEM
!!
!!    PURPOSE
!!    -------
!     Declaration of surface parameters
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	B. Bueno   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       10/2010
!!      G. Pigeon      06/2011 add LSHAD_DAY
!!      G. Pigeon      07/2011 add LNATVENT_NIGHT
!!      G. Pigeon      08/2011 change from MODD_BLD -> MODD_BEM
!!      G. Pigeon      10/2011 add indoor relative surf. and view factors
!!      G. Pigeon      09/2012 add TRAN_WIN
!!      G. Pigeon      10/2012 add XF_WIN_WIN
!
!*       0.   DECLARATIONS
!             ------------
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE BEM_OPTIONS_t
! BLD scheme option
!
! Number of layers
!
  INTEGER                       :: NFLOOR_LAYER   ! number of layers in walls
  CHARACTER(LEN=6)              :: CCOOL_COIL    ! type of cooling coil
  CHARACTER(LEN=6)              :: CHEAT_COIL    ! type of heating coil
  LOGICAL                       :: LAUTOSIZE     ! Flag to activate autosize calculations
!
END TYPE BEM_OPTIONS_t
!
TYPE(BEM_OPTIONS_t), ALLOCATABLE, TARGET, SAVE :: BEM_OPTIONS_MODEL(:)
!
INTEGER, POINTER :: NFLOOR_LAYER=>NULL()
!$OMP THREADPRIVATE(NFLOOR_LAYER)
 CHARACTER(LEN=6), POINTER     :: CCOOL_COIL=>NULL()
!$OMP THREADPRIVATE(CCOOL_COIL)
 CHARACTER(LEN=6), POINTER     :: CHEAT_COIL=>NULL()
!$OMP THREADPRIVATE(CHEAT_COIL)
LOGICAL, POINTER :: LAUTOSIZE=>NULL()
!$OMP THREADPRIVATE(LAUTOSIZE)
!
!
!--------------------------------------------------------------------------
!
TYPE BEM_t
!
! Floor parameters
!
  REAL, POINTER, DIMENSION(:,:) :: XHC_FLOOR     ! floor layers heat capacity        (J/K/m3)
  REAL, POINTER, DIMENSION(:,:) :: XTC_FLOOR     ! floor layers thermal conductivity (W/K/m)
  REAL, POINTER, DIMENSION(:,:) :: XD_FLOOR      ! depth of floor layers             (m)
!
! HVAC parameters
!
  REAL, POINTER, DIMENSION(:)   :: XTCOOL_TARGET ! cooling setpoint of indoor air
  REAL, POINTER, DIMENSION(:)   :: XTHEAT_TARGET ! heating setpoint of indoor air
  REAL, POINTER, DIMENSION(:)   :: XF_WASTE_CAN  ! fraction of waste heat released into the canyon
  REAL, POINTER, DIMENSION(:)   :: XEFF_HEAT     ! efficiency of the heating system
!
! Indoor parameters
!
  REAL, POINTER, DIMENSION(:)   :: XTI_BLD       ! building interior temperature    (K)
  REAL, POINTER, DIMENSION(:,:) :: XT_FLOOR      ! floor layer temperatures         (K)
  REAL, POINTER, DIMENSION(:,:) :: XT_MASS       ! Air cooled building internal th. mass temperature (K)
!
  REAL, POINTER, DIMENSION(:)   :: XQIN          ! internal heat gains [W m-2(floor)]
  REAL, POINTER, DIMENSION(:)   :: XQIN_FRAD     ! radiant fraction of internal heat gains
  REAL, POINTER, DIMENSION(:)   :: XSHGC         ! solar heat gain coef. of windows
  REAL, POINTER, DIMENSION(:)   :: XSHGC_SH      ! solar heat gain coef. of windows + shading
  REAL, POINTER, DIMENSION(:)   :: XU_WIN        ! window U-factor [K m W-2]
  REAL, POINTER, DIMENSION(:)   :: XTRAN_WIN     ! window transmittance (-)
  REAL, POINTER, DIMENSION(:)   :: XGR           ! glazing ratio
  REAL, POINTER, DIMENSION(:)   :: XFLOOR_HEIGHT ! building floor height [m]
  REAL, POINTER, DIMENSION(:)   :: XINF          ! infiltration/ventilation flow rate [AC/H]
!
! New parameters
!
  REAL, POINTER, DIMENSION(:)   :: XF_WATER_COND  ! fraction of evaporation for condensers (cooling system)
  REAL, POINTER, DIMENSION(:)   :: XAUX_MAX      ! Auxiliar variable for autosize calcs
  REAL, POINTER, DIMENSION(:)   :: XQIN_FLAT     ! Latent franction of internal heat gains
  REAL, POINTER, DIMENSION(:)   :: XHR_TARGET    ! Relative humidity setpoint
  REAL, POINTER, DIMENSION(:)   :: XT_WIN2       ! Indoor window temperature [K]
  REAL, POINTER, DIMENSION(:)   :: XQI_BLD       ! Indoor air specific humidity [kg kg-1]
  REAL, POINTER, DIMENSION(:)   :: XV_VENT       ! Ventilation flow rate [AC/H]
  REAL, POINTER, DIMENSION(:)   :: XCAP_SYS_HEAT ! Capacity of the heating system 
                                                 ! [W m-2(bld)]
  REAL, POINTER, DIMENSION(:)   :: XCAP_SYS_RAT  ! Rated capacity of the cooling system
                                                 ! [W m-2(bld)]
  REAL, POINTER, DIMENSION(:)   :: XT_ADP        ! Apparatus dewpoint temperature of the
                                                 ! cooling coil [K]
  REAL, POINTER, DIMENSION(:)   :: XM_SYS_RAT    ! Rated HVAC mass flow rate 
                                                 ! [kg s-1 m-2(bld)]
  REAL, POINTER, DIMENSION(:)   :: XCOP_RAT      ! Rated COP of the cooling system
  REAL, POINTER, DIMENSION(:)   :: XT_WIN1       ! outdoor window temperature [K]
  REAL, POINTER, DIMENSION(:)   :: XALB_WIN      ! window albedo
  REAL, POINTER, DIMENSION(:)   :: XABS_WIN      ! window absortance
  REAL, POINTER, DIMENSION(:)   :: XT_SIZE_MAX   ! Maximum outdoor air temperature for
                                                 ! HVAC sizing [K]
  REAL, POINTER, DIMENSION(:)   :: XT_SIZE_MIN   ! Minimum outdoor air temperature for
                                                 ! HVAC sizing [K]
  REAL, POINTER, DIMENSION(:)   :: XUGG_WIN      ! Window glass-to-glass U-factor [K m W-2]
  LOGICAL, POINTER, DIMENSION(:):: LSHADE        ! flag to activate shading devices -> LOGICAL in the code
  REAL,    POINTER, DIMENSION(:):: XSHADE        ! flag to activate shading devices -> REAL for i/o 0. or 1.
  CHARACTER(LEN=4), POINTER, DIMENSION(:) :: CNATVENT ! flag to activate natural ventilation 'NONE', 'MANU', 'AUTO'
  REAL,    POINTER, DIMENSION(:):: XNATVENT      ! flag to describe surventilation system for i/o 
                                                 ! 0 for NONE, 1 for MANU and 2 for AUTO
  LOGICAL, POINTER, DIMENSION(:):: LSHAD_DAY     !Has shading been necessary this day ?
  LOGICAL, POINTER, DIMENSION(:):: LNATVENT_NIGHT !Has nocturnal surventilation been necessary and possible this night ?
  !
  !indoor relative surfaces and view factors
  REAL, POINTER, DIMENSION(:) :: XN_FLOOR        ! Number of floors     
  REAL, POINTER, DIMENSION(:) :: XGLAZ_O_BLD    ! Window area [m2_win/m2_bld]
  REAL, POINTER, DIMENSION(:) :: XMASS_O_BLD    ! Mass area [m2_mass/m2_bld]
  REAL, POINTER, DIMENSION(:) :: XFLOOR_HW_RATIO ! H/W ratio of 1 floor level
  REAL, POINTER, DIMENSION(:) :: XF_FLOOR_MASS   ! View factor floor-mass
  REAL, POINTER, DIMENSION(:) :: XF_FLOOR_WALL   ! View factor floor-wall
  REAL, POINTER, DIMENSION(:) :: XF_FLOOR_WIN    ! View factor floor-window
  REAL, POINTER, DIMENSION(:) :: XF_FLOOR_ROOF   ! View factor floor-roof
  REAL, POINTER, DIMENSION(:) :: XF_WALL_FLOOR   ! View factor wall-floor
  REAL, POINTER, DIMENSION(:) :: XF_WALL_MASS    ! View factor wall-mass
  REAL, POINTER, DIMENSION(:) :: XF_WALL_WIN     ! View factor wall-win
  REAL, POINTER, DIMENSION(:) :: XF_WIN_FLOOR    ! View factor win-floor
  REAL, POINTER, DIMENSION(:) :: XF_WIN_MASS     ! View factor win-mass
  REAL, POINTER, DIMENSION(:) :: XF_WIN_WALL     ! View factor win-wall
  REAL, POINTER, DIMENSION(:) :: XF_WIN_WIN      ! indoor View factor win-win
  REAL, POINTER, DIMENSION(:) :: XF_MASS_FLOOR   ! View factor mass-floor
  REAL, POINTER, DIMENSION(:) :: XF_MASS_WALL    ! View factor mass-wall
  REAL, POINTER, DIMENSION(:) :: XF_MASS_WIN     ! View factor mass-window


! 
END TYPE BEM_t

TYPE(BEM_t), ALLOCATABLE, TARGET, SAVE :: BEM_MODEL(:,:)

REAL, POINTER, DIMENSION(:,:) :: XHC_FLOOR=>NULL()
!$OMP THREADPRIVATE(XHC_FLOOR)
REAL, POINTER, DIMENSION(:,:) :: XTC_FLOOR=>NULL()
!$OMP THREADPRIVATE(XTC_FLOOR)
REAL, POINTER, DIMENSION(:,:) :: XD_FLOOR=>NULL()
!$OMP THREADPRIVATE(XD_FLOOR)
REAL, POINTER, DIMENSION(:)   :: XTCOOL_TARGET=>NULL()
!$OMP THREADPRIVATE(XTCOOL_TARGET)
REAL, POINTER, DIMENSION(:)   :: XTHEAT_TARGET=>NULL()
!$OMP THREADPRIVATE(XTHEAT_TARGET)
REAL, POINTER, DIMENSION(:)   :: XTI_BLD=>NULL()
!$OMP THREADPRIVATE(XTI_BLD)
REAL, POINTER, DIMENSION(:,:) :: XT_FLOOR=>NULL()
!$OMP THREADPRIVATE(XT_FLOOR)
REAL, POINTER, DIMENSION(:)   :: XQIN=>NULL()
!$OMP THREADPRIVATE(XQIN)
REAL, POINTER, DIMENSION(:)   :: XQIN_FRAD=>NULL()
!$OMP THREADPRIVATE(XQIN_FRAD)
REAL, POINTER, DIMENSION(:)   :: XSHGC=>NULL()
!$OMP THREADPRIVATE(XSHGC)
REAL, POINTER, DIMENSION(:)   :: XSHGC_SH=>NULL()
!$OMP THREADPRIVATE(XSHGC_SH)
REAL, POINTER, DIMENSION(:)   :: XU_WIN=>NULL()
!$OMP THREADPRIVATE(XU_WIN)
REAL, POINTER, DIMENSION(:)   :: XTRAN_WIN=>NULL()
!$OMP THREADPRIVATE(XTRAN_WIN)
REAL, POINTER, DIMENSION(:)   :: XGR=>NULL()
!$OMP THREADPRIVATE(XGR)
REAL, POINTER, DIMENSION(:)   :: XFLOOR_HEIGHT=>NULL()
!$OMP THREADPRIVATE(XFLOOR_HEIGHT)
REAL, POINTER, DIMENSION(:)   :: XEFF_HEAT=>NULL()
!$OMP THREADPRIVATE(XEFF_HEAT)
REAL, POINTER, DIMENSION(:)   :: XINF=>NULL()
!$OMP THREADPRIVATE(XINF)
REAL, POINTER, DIMENSION(:,:) :: XT_MASS=>NULL()
!$OMP THREADPRIVATE(XT_MASS)
REAL, POINTER, DIMENSION(:)   :: XF_WASTE_CAN=>NULL()
!$OMP THREADPRIVATE(XF_WASTE_CAN)
!
LOGICAL, POINTER, DIMENSION(:) :: LSHADE=>NULL()
!$OMP THREADPRIVATE(LSHADE)
REAL, POINTER, DIMENSION(:) :: XSHADE=>NULL()
!$OMP THREADPRIVATE(XSHADE)
 CHARACTER(LEN=4), POINTER, DIMENSION(:) :: CNATVENT=>NULL()
!$OMP THREADPRIVATE(CNATVENT)
REAL, POINTER, DIMENSION(:) :: XNATVENT=>NULL()
!$OMP THREADPRIVATE(XNATVENT)
REAL, POINTER, DIMENSION(:)   :: XF_WATER_COND=>NULL()
!$OMP THREADPRIVATE(XF_WATER_COND)
REAL, POINTER, DIMENSION(:)   :: XAUX_MAX=>NULL()
!$OMP THREADPRIVATE(XAUX_MAX)
REAL, POINTER, DIMENSION(:)   :: XQIN_FLAT=>NULL()
!$OMP THREADPRIVATE(XQIN_FLAT)
REAL, POINTER, DIMENSION(:)   :: XHR_TARGET=>NULL()
!$OMP THREADPRIVATE(XHR_TARGET)
REAL, POINTER, DIMENSION(:)   :: XT_WIN2=>NULL()
!$OMP THREADPRIVATE(XT_WIN2)
REAL, POINTER, DIMENSION(:)   :: XQI_BLD=>NULL()
!$OMP THREADPRIVATE(XQI_BLD)
REAL, POINTER, DIMENSION(:)   :: XV_VENT=>NULL()
!$OMP THREADPRIVATE(XV_VENT)
REAL, POINTER, DIMENSION(:)   :: XCAP_SYS_HEAT=>NULL()
!$OMP THREADPRIVATE(XCAP_SYS_HEAT)
REAL, POINTER, DIMENSION(:)   :: XCAP_SYS_RAT=>NULL()
!$OMP THREADPRIVATE(XCAP_SYS_RAT)
REAL, POINTER, DIMENSION(:)   :: XT_ADP=>NULL()
!$OMP THREADPRIVATE(XT_ADP)
REAL, POINTER, DIMENSION(:)   :: XM_SYS_RAT=>NULL()
!$OMP THREADPRIVATE(XM_SYS_RAT)
REAL, POINTER, DIMENSION(:)   :: XCOP_RAT=>NULL()
!$OMP THREADPRIVATE(XCOP_RAT)
REAL, POINTER, DIMENSION(:)   :: XT_WIN1=>NULL()
!$OMP THREADPRIVATE(XT_WIN1)
REAL, POINTER, DIMENSION(:)   :: XALB_WIN=>NULL()
!$OMP THREADPRIVATE(XALB_WIN)
REAL, POINTER, DIMENSION(:)   :: XABS_WIN=>NULL()
!$OMP THREADPRIVATE(XABS_WIN)
REAL, POINTER, DIMENSION(:)   :: XT_SIZE_MAX=>NULL()
!$OMP THREADPRIVATE(XT_SIZE_MAX)
REAL, POINTER, DIMENSION(:)   :: XT_SIZE_MIN=>NULL()
!$OMP THREADPRIVATE(XT_SIZE_MIN)
REAL, POINTER, DIMENSION(:)   :: XUGG_WIN=>NULL()
!$OMP THREADPRIVATE(XUGG_WIN)
LOGICAL, POINTER, DIMENSION(:):: LSHAD_DAY=>NULL()
!$OMP THREADPRIVATE(LSHAD_DAY)
LOGICAL, POINTER, DIMENSION(:):: LNATVENT_NIGHT =>NULL()
!$OMP THREADPRIVATE(LNATVENT_NIGHT)
REAL, POINTER, DIMENSION(:) :: XN_FLOOR=> NULL()
!$OMP THREADPRIVATE(XN_FLOOR)
REAL, POINTER, DIMENSION(:) :: XGLAZ_O_BLD=> NULL()
!$OMP THREADPRIVATE(XGLAZ_O_BLD)
REAL, POINTER, DIMENSION(:) :: XMASS_O_BLD=> NULL()
!$OMP THREADPRIVATE(XMASS_O_BLD)
REAL, POINTER, DIMENSION(:) :: XFLOOR_HW_RATIO=> NULL()
!$OMP THREADPRIVATE(XFLOOR_HW_RATIO)
REAL, POINTER, DIMENSION(:) :: XF_FLOOR_MASS=> NULL()
!$OMP THREADPRIVATE(XF_FLOOR_MASS)
REAL, POINTER, DIMENSION(:) :: XF_FLOOR_WALL=> NULL()
!$OMP THREADPRIVATE(XF_FLOOR_WALL)
REAL, POINTER, DIMENSION(:) :: XF_FLOOR_WIN=> NULL()
!$OMP THREADPRIVATE(XF_FLOOR_WIN)
REAL, POINTER, DIMENSION(:) :: XF_FLOOR_ROOF=> NULL()
!$OMP THREADPRIVATE(XF_FLOOR_ROOF)
REAL, POINTER, DIMENSION(:) :: XF_WALL_FLOOR=> NULL()
!$OMP THREADPRIVATE(XF_WALL_FLOOR)
REAL, POINTER, DIMENSION(:) :: XF_WALL_MASS=> NULL()
!$OMP THREADPRIVATE(XF_WALL_MASS)
REAL, POINTER, DIMENSION(:) :: XF_WALL_WIN=> NULL()
!$OMP THREADPRIVATE(XF_WALL_WIN)
REAL, POINTER, DIMENSION(:) :: XF_WIN_FLOOR=> NULL()
!$OMP THREADPRIVATE(XF_WIN_FLOOR)
REAL, POINTER, DIMENSION(:) :: XF_WIN_MASS=> NULL()
!$OMP THREADPRIVATE(XF_WIN_MASS)
REAL, POINTER, DIMENSION(:) :: XF_WIN_WALL=> NULL()
!$OMP THREADPRIVATE(XF_WIN_WALL)
REAL, POINTER, DIMENSION(:) :: XF_WIN_WIN=> NULL()
!$OMP THREADPRIVATE(XF_WIN_WIN)
REAL, POINTER, DIMENSION(:) :: XF_MASS_FLOOR=> NULL()
!$OMP THREADPRIVATE(XF_MASS_FLOOR)
REAL, POINTER, DIMENSION(:) :: XF_MASS_WALL=> NULL()
!$OMP THREADPRIVATE(XF_MASS_WALL)
REAL, POINTER, DIMENSION(:) :: XF_MASS_WIN=> NULL()
!$OMP THREADPRIVATE(XF_MASS_WIN)

CONTAINS

SUBROUTINE BEM_OPTIONS_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_BEM_N:BEM_OPTIONS_GOTO_MODEL',0,ZHOOK_HANDLE)
NFLOOR_LAYER=>BEM_OPTIONS_MODEL(KTO)%NFLOOR_LAYER
CCOOL_COIL=>BEM_OPTIONS_MODEL(KTO)%CCOOL_COIL
CHEAT_COIL=>BEM_OPTIONS_MODEL(KTO)%CHEAT_COIL
LAUTOSIZE=>BEM_OPTIONS_MODEL(KTO)%LAUTOSIZE
IF (LHOOK) CALL DR_HOOK('MODD_BEM_N:BEM_OPTIONS_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE BEM_OPTIONS_GOTO_MODEL

SUBROUTINE BEM_OPTIONS_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_BEM_N:BEM_OPTIONS_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(BEM_OPTIONS_MODEL(KMODEL))
BEM_OPTIONS_MODEL(:)%NFLOOR_LAYER = 0
BEM_OPTIONS_MODEL(:)%CCOOL_COIL   = '      '
BEM_OPTIONS_MODEL(:)%CHEAT_COIL   = '      '
BEM_OPTIONS_MODEL(:)%LAUTOSIZE    = .FALSE.
IF (LHOOK) CALL DR_HOOK("MODD_BEM_N:BEM_OPTIONS_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE BEM_OPTIONS_ALLOC

SUBROUTINE BEM_OPTIONS_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_BEM_N:BEM_OPTIONS_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(BEM_OPTIONS_MODEL)) DEALLOCATE(BEM_OPTIONS_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_BEM_N:BEM_OPTIONS_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE BEM_OPTIONS_DEALLO

!----------------------------------------------------------------------------

SUBROUTINE BEM_GOTO_MODEL(KFROM, KTO, LKFROM, KFROM_PATCH, KTO_PATCH)
INTEGER, INTENT(IN) :: KFROM, KTO
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM_PATCH, KTO_PATCH
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
   BEM_MODEL(KFROM,KFROM_PATCH)%LSHADE=>LSHADE
   BEM_MODEL(KFROM,KFROM_PATCH)%XSHADE=>XSHADE
   BEM_MODEL(KFROM,KFROM_PATCH)%CNATVENT=>CNATVENT
   BEM_MODEL(KFROM,KFROM_PATCH)%XNATVENT=>XNATVENT
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_WATER_COND=>XF_WATER_COND
   BEM_MODEL(KFROM,KFROM_PATCH)%XHC_FLOOR=>XHC_FLOOR
   BEM_MODEL(KFROM,KFROM_PATCH)%XTC_FLOOR=>XTC_FLOOR
   BEM_MODEL(KFROM,KFROM_PATCH)%XD_FLOOR=>XD_FLOOR
   BEM_MODEL(KFROM,KFROM_PATCH)%XTCOOL_TARGET=>XTCOOL_TARGET
   BEM_MODEL(KFROM,KFROM_PATCH)%XTHEAT_TARGET=>XTHEAT_TARGET
   BEM_MODEL(KFROM,KFROM_PATCH)%XTI_BLD=>XTI_BLD
   BEM_MODEL(KFROM,KFROM_PATCH)%XT_FLOOR=>XT_FLOOR
   BEM_MODEL(KFROM,KFROM_PATCH)%XT_MASS=>XT_MASS
   BEM_MODEL(KFROM,KFROM_PATCH)%XQIN=>XQIN
   BEM_MODEL(KFROM,KFROM_PATCH)%XQIN_FRAD=>XQIN_FRAD
   BEM_MODEL(KFROM,KFROM_PATCH)%XSHGC=>XSHGC
   BEM_MODEL(KFROM,KFROM_PATCH)%XSHGC_SH=>XSHGC_SH
   BEM_MODEL(KFROM,KFROM_PATCH)%XU_WIN=>XU_WIN
   BEM_MODEL(KFROM,KFROM_PATCH)%XTRAN_WIN=>XTRAN_WIN
   BEM_MODEL(KFROM,KFROM_PATCH)%XGR=>XGR
   BEM_MODEL(KFROM,KFROM_PATCH)%XFLOOR_HEIGHT=>XFLOOR_HEIGHT
   BEM_MODEL(KFROM,KFROM_PATCH)%XEFF_HEAT=>XEFF_HEAT
   BEM_MODEL(KFROM,KFROM_PATCH)%XINF=>XINF
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_WASTE_CAN=>XF_WASTE_CAN
   !
   BEM_MODEL(KFROM,KFROM_PATCH)%XAUX_MAX=>XAUX_MAX
   BEM_MODEL(KFROM,KFROM_PATCH)%XQIN_FLAT=>XQIN_FLAT
   BEM_MODEL(KFROM,KFROM_PATCH)%XHR_TARGET=>XHR_TARGET
   BEM_MODEL(KFROM,KFROM_PATCH)%XT_WIN2=>XT_WIN2
   BEM_MODEL(KFROM,KFROM_PATCH)%XQI_BLD=>XQI_BLD
   BEM_MODEL(KFROM,KFROM_PATCH)%XV_VENT=>XV_VENT
   BEM_MODEL(KFROM,KFROM_PATCH)%XCAP_SYS_HEAT=>XCAP_SYS_HEAT
   BEM_MODEL(KFROM,KFROM_PATCH)%XCAP_SYS_RAT=>XCAP_SYS_RAT
   BEM_MODEL(KFROM,KFROM_PATCH)%XT_ADP=>XT_ADP
   BEM_MODEL(KFROM,KFROM_PATCH)%XM_SYS_RAT=>XM_SYS_RAT
   BEM_MODEL(KFROM,KFROM_PATCH)%XCOP_RAT=>XCOP_RAT
   BEM_MODEL(KFROM,KFROM_PATCH)%XT_WIN1=>XT_WIN1
   BEM_MODEL(KFROM,KFROM_PATCH)%XALB_WIN=>XALB_WIN
   BEM_MODEL(KFROM,KFROM_PATCH)%XABS_WIN=>XABS_WIN
   BEM_MODEL(KFROM,KFROM_PATCH)%XT_SIZE_MAX=>XT_SIZE_MAX
   BEM_MODEL(KFROM,KFROM_PATCH)%XT_SIZE_MIN=>XT_SIZE_MIN
   BEM_MODEL(KFROM,KFROM_PATCH)%XUGG_WIN=>XUGG_WIN
   BEM_MODEL(KFROM,KFROM_PATCH)%LSHAD_DAY=>LSHAD_DAY
   BEM_MODEL(KFROM,KFROM_PATCH)%LNATVENT_NIGHT=>LNATVENT_NIGHT
   !
   BEM_MODEL(KFROM,KFROM_PATCH)%XN_FLOOR=>XN_FLOOR
   BEM_MODEL(KFROM,KFROM_PATCH)%XGLAZ_O_BLD=>XGLAZ_O_BLD
   BEM_MODEL(KFROM,KFROM_PATCH)%XMASS_O_BLD=>XMASS_O_BLD
   BEM_MODEL(KFROM,KFROM_PATCH)%XFLOOR_HW_RATIO=>XFLOOR_HW_RATIO
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_FLOOR_MASS=>XF_FLOOR_MASS
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_FLOOR_WALL=>XF_FLOOR_WALL
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_FLOOR_WIN=>XF_FLOOR_WIN
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_FLOOR_ROOF=>XF_FLOOR_ROOF
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_WALL_FLOOR=>XF_WALL_FLOOR
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_WALL_MASS=>XF_WALL_MASS
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_WALL_WIN=>XF_WALL_WIN
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_WIN_FLOOR=>XF_WIN_FLOOR
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_WIN_MASS=>XF_WIN_MASS
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_WIN_WALL=>XF_WIN_WALL
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_WIN_WIN=>XF_WIN_WIN
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_MASS_FLOOR=>XF_MASS_FLOOR
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_MASS_WALL=>XF_MASS_WALL
   BEM_MODEL(KFROM,KFROM_PATCH)%XF_MASS_WIN=>XF_MASS_WIN
ENDIF
!
IF (LHOOK) CALL DR_HOOK('MODD_BEM_N:BEM_GOTO_MODEL',0,ZHOOK_HANDLE)
!
XHC_FLOOR=>BEM_MODEL(KTO,KTO_PATCH)%XHC_FLOOR
XTC_FLOOR=>BEM_MODEL(KTO,KTO_PATCH)%XTC_FLOOR
XD_FLOOR=>BEM_MODEL(KTO,KTO_PATCH)%XD_FLOOR
XTCOOL_TARGET=>BEM_MODEL(KTO,KTO_PATCH)%XTCOOL_TARGET
XTHEAT_TARGET=>BEM_MODEL(KTO,KTO_PATCH)%XTHEAT_TARGET
XTI_BLD=>BEM_MODEL(KTO,KTO_PATCH)%XTI_BLD
XT_FLOOR=>BEM_MODEL(KTO,KTO_PATCH)%XT_FLOOR
XT_MASS=>BEM_MODEL(KTO,KTO_PATCH)%XT_MASS
XQIN=>BEM_MODEL(KTO,KTO_PATCH)%XQIN
XQIN_FRAD=>BEM_MODEL(KTO,KTO_PATCH)%XQIN_FRAD
XSHGC=>BEM_MODEL(KTO,KTO_PATCH)%XSHGC
XSHGC_SH=>BEM_MODEL(KTO,KTO_PATCH)%XSHGC_SH
XU_WIN=>BEM_MODEL(KTO,KTO_PATCH)%XU_WIN
XTRAN_WIN=>BEM_MODEL(KTO,KTO_PATCH)%XTRAN_WIN
XGR=>BEM_MODEL(KTO,KTO_PATCH)%XGR
XFLOOR_HEIGHT=>BEM_MODEL(KTO,KTO_PATCH)%XFLOOR_HEIGHT
XEFF_HEAT=>BEM_MODEL(KTO,KTO_PATCH)%XEFF_HEAT
XINF=>BEM_MODEL(KTO,KTO_PATCH)%XINF
XF_WASTE_CAN=>BEM_MODEL(KTO,KTO_PATCH)%XF_WASTE_CAN
!
LSHADE=>BEM_MODEL(KTO,KTO_PATCH)%LSHADE
XSHADE=>BEM_MODEL(KTO,KTO_PATCH)%XSHADE
CNATVENT=>BEM_MODEL(KTO,KTO_PATCH)%CNATVENT
XNATVENT=>BEM_MODEL(KTO,KTO_PATCH)%XNATVENT
XF_WATER_COND=>BEM_MODEL(KTO,KTO_PATCH)%XF_WATER_COND
XAUX_MAX=>BEM_MODEL(KTO,KTO_PATCH)%XAUX_MAX
XQIN_FLAT=>BEM_MODEL(KTO,KTO_PATCH)%XQIN_FLAT
XHR_TARGET=>BEM_MODEL(KTO,KTO_PATCH)%XHR_TARGET
XT_WIN2=>BEM_MODEL(KTO,KTO_PATCH)%XT_WIN2
XQI_BLD=>BEM_MODEL(KTO,KTO_PATCH)%XQI_BLD
XV_VENT=>BEM_MODEL(KTO,KTO_PATCH)%XV_VENT
XCAP_SYS_HEAT=>BEM_MODEL(KTO,KTO_PATCH)%XCAP_SYS_HEAT
XCAP_SYS_RAT=>BEM_MODEL(KTO,KTO_PATCH)%XCAP_SYS_RAT
XT_ADP=>BEM_MODEL(KTO,KTO_PATCH)%XT_ADP
XM_SYS_RAT=>BEM_MODEL(KTO,KTO_PATCH)%XM_SYS_RAT
XCOP_RAT=>BEM_MODEL(KTO,KTO_PATCH)%XCOP_RAT
XT_WIN1=>BEM_MODEL(KTO,KTO_PATCH)%XT_WIN1
XALB_WIN=>BEM_MODEL(KTO,KTO_PATCH)%XALB_WIN
XABS_WIN=>BEM_MODEL(KTO,KTO_PATCH)%XABS_WIN
XT_SIZE_MAX=>BEM_MODEL(KTO,KTO_PATCH)%XT_SIZE_MAX
XT_SIZE_MIN=>BEM_MODEL(KTO,KTO_PATCH)%XT_SIZE_MIN
XUGG_WIN=>BEM_MODEL(KTO,KTO_PATCH)%XUGG_WIN
LSHAD_DAY=>BEM_MODEL(KTO,KTO_PATCH)%LSHAD_DAY
LNATVENT_NIGHT=>BEM_MODEL(KTO,KTO_PATCH)%LNATVENT_NIGHT
!
XN_FLOOR=>BEM_MODEL(KTO,KTO_PATCH)%XN_FLOOR
XGLAZ_O_BLD=>BEM_MODEL(KTO,KTO_PATCH)%XGLAZ_O_BLD
XMASS_O_BLD=>BEM_MODEL(KTO,KTO_PATCH)%XMASS_O_BLD
XFLOOR_HW_RATIO=>BEM_MODEL(KTO,KTO_PATCH)%XFLOOR_HW_RATIO
XF_FLOOR_MASS=>BEM_MODEL(KTO,KTO_PATCH)%XF_FLOOR_MASS
XF_FLOOR_WALL=>BEM_MODEL(KTO,KTO_PATCH)%XF_FLOOR_WALL
XF_FLOOR_WIN=>BEM_MODEL(KTO,KTO_PATCH)%XF_FLOOR_WIN
XF_FLOOR_ROOF=>BEM_MODEL(KTO,KTO_PATCH)%XF_FLOOR_ROOF
XF_WALL_FLOOR=>BEM_MODEL(KTO,KTO_PATCH)%XF_WALL_FLOOR
XF_WALL_MASS=>BEM_MODEL(KTO,KTO_PATCH)%XF_WALL_MASS
XF_WALL_WIN=>BEM_MODEL(KTO,KTO_PATCH)%XF_WALL_WIN
XF_WIN_FLOOR=>BEM_MODEL(KTO,KTO_PATCH)%XF_WIN_FLOOR
XF_WIN_MASS=>BEM_MODEL(KTO,KTO_PATCH)%XF_WIN_MASS
XF_WIN_WALL=>BEM_MODEL(KTO,KTO_PATCH)%XF_WIN_WALL
XF_WIN_WIN=>BEM_MODEL(KTO,KTO_PATCH)%XF_WIN_WIN
XF_MASS_FLOOR=>BEM_MODEL(KTO,KTO_PATCH)%XF_MASS_FLOOR
XF_MASS_WALL=>BEM_MODEL(KTO,KTO_PATCH)%XF_MASS_WALL
XF_MASS_WIN=>BEM_MODEL(KTO,KTO_PATCH)%XF_MASS_WIN
!
IF (LHOOK) CALL DR_HOOK('MODD_BEM_N:BEM_GOTO_MODEL',1,ZHOOK_HANDLE)
!
END SUBROUTINE BEM_GOTO_MODEL
!
!
SUBROUTINE BEM_ALLOC(KMODEL,KPATCH)
INTEGER, INTENT(IN) :: KMODEL
INTEGER, INTENT(IN) :: KPATCH
INTEGER :: J,JP
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_BEM_N:BEM_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(BEM_MODEL(KMODEL,KPATCH))
DO J=1,KMODEL
 DO JP=1,KPATCH
  NULLIFY(BEM_MODEL(J,JP)%XF_WATER_COND)
  NULLIFY(BEM_MODEL(J,JP)%XHC_FLOOR)
  NULLIFY(BEM_MODEL(J,JP)%XTC_FLOOR)
  NULLIFY(BEM_MODEL(J,JP)%XD_FLOOR)
  NULLIFY(BEM_MODEL(J,JP)%XTCOOL_TARGET)
  NULLIFY(BEM_MODEL(J,JP)%XTHEAT_TARGET)
  NULLIFY(BEM_MODEL(J,JP)%XTI_BLD)
  NULLIFY(BEM_MODEL(J,JP)%XT_FLOOR)
  NULLIFY(BEM_MODEL(J,JP)%XT_MASS)
  NULLIFY(BEM_MODEL(J,JP)%XQIN)
  NULLIFY(BEM_MODEL(J,JP)%XQIN_FRAD)
  NULLIFY(BEM_MODEL(J,JP)%XSHGC)
  NULLIFY(BEM_MODEL(J,JP)%XSHGC_SH)
  NULLIFY(BEM_MODEL(J,JP)%XU_WIN)
  NULLIFY(BEM_MODEL(J,JP)%XTRAN_WIN)
  NULLIFY(BEM_MODEL(J,JP)%XGR)
  NULLIFY(BEM_MODEL(J,JP)%XFLOOR_HEIGHT)
  NULLIFY(BEM_MODEL(J,JP)%XEFF_HEAT)
  NULLIFY(BEM_MODEL(J,JP)%XINF)
  NULLIFY(BEM_MODEL(J,JP)%XF_WASTE_CAN)
  NULLIFY(BEM_MODEL(J,JP)%XAUX_MAX)
  NULLIFY(BEM_MODEL(J,JP)%XQIN_FLAT)
  NULLIFY(BEM_MODEL(J,JP)%XHR_TARGET)
  NULLIFY(BEM_MODEL(J,JP)%XT_WIN2)
  NULLIFY(BEM_MODEL(J,JP)%XQI_BLD)
  NULLIFY(BEM_MODEL(J,JP)%XV_VENT)
  NULLIFY(BEM_MODEL(J,JP)%XCAP_SYS_HEAT)
  NULLIFY(BEM_MODEL(J,JP)%XCAP_SYS_RAT)
  NULLIFY(BEM_MODEL(J,JP)%XT_ADP)
  NULLIFY(BEM_MODEL(J,JP)%XM_SYS_RAT)
  NULLIFY(BEM_MODEL(J,JP)%XCOP_RAT)
  NULLIFY(BEM_MODEL(J,JP)%XT_WIN1)
  NULLIFY(BEM_MODEL(J,JP)%XALB_WIN)
  NULLIFY(BEM_MODEL(J,JP)%XABS_WIN)
  NULLIFY(BEM_MODEL(J,JP)%XT_SIZE_MAX)
  NULLIFY(BEM_MODEL(J,JP)%XT_SIZE_MIN)
  NULLIFY(BEM_MODEL(J,JP)%XUGG_WIN)
  NULLIFY(BEM_MODEL(J,JP)%LSHAD_DAY)
  NULLIFY(BEM_MODEL(J,JP)%LNATVENT_NIGHT)
  NULLIFY(BEM_MODEL(J,JP)%LSHADE)
  NULLIFY(BEM_MODEL(J,JP)%XSHADE)
  NULLIFY(BEM_MODEL(J,JP)%CNATVENT)
  NULLIFY(BEM_MODEL(J,JP)%XNATVENT)
  NULLIFY(BEM_MODEL(J,JP)%XN_FLOOR)
  NULLIFY(BEM_MODEL(J,JP)%XGLAZ_O_BLD)
  NULLIFY(BEM_MODEL(J,JP)%XMASS_O_BLD)
  NULLIFY(BEM_MODEL(J,JP)%XFLOOR_HW_RATIO)
  NULLIFY(BEM_MODEL(J,JP)%XF_FLOOR_MASS)
  NULLIFY(BEM_MODEL(J,JP)%XF_FLOOR_WALL)
  NULLIFY(BEM_MODEL(J,JP)%XF_FLOOR_WIN)
  NULLIFY(BEM_MODEL(J,JP)%XF_FLOOR_ROOF)
  NULLIFY(BEM_MODEL(J,JP)%XF_WALL_FLOOR)
  NULLIFY(BEM_MODEL(J,JP)%XF_WALL_MASS)
  NULLIFY(BEM_MODEL(J,JP)%XF_WALL_WIN)
  NULLIFY(BEM_MODEL(J,JP)%XF_WIN_FLOOR)
  NULLIFY(BEM_MODEL(J,JP)%XF_WIN_MASS)
  NULLIFY(BEM_MODEL(J,JP)%XF_WIN_WALL)
  NULLIFY(BEM_MODEL(J,JP)%XF_WIN_WIN)
  NULLIFY(BEM_MODEL(J,JP)%XF_MASS_FLOOR)
  NULLIFY(BEM_MODEL(J,JP)%XF_MASS_WALL)
  NULLIFY(BEM_MODEL(J,JP)%XF_MASS_WIN)
 ENDDO
ENDDO
IF (LHOOK) CALL DR_HOOK("MODD_BEM_N:BEM_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE BEM_ALLOC
!
SUBROUTINE BEM_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_BEM_N:BEM_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(BEM_MODEL)) DEALLOCATE(BEM_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_BEM_N:BEM_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE BEM_DEALLO
!
END MODULE MODD_BEM_n
