!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ##########################
      MODULE MODD_DATA_TSZ0_n
!     ##########################
!
!!****  *MODD_DATA_TSZ0 - declaration of TSZ0 surface parameters for TSZ0 scheme
!!
!!    PURPOSE
!!    -------
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	P. Le Moigne  *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       09/2007
!
!*       0.   DECLARATIONS
!             ------------
!
!
    
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE DATA_TSZ0_t
!-------------------------------------------------------------------------------
INTEGER                     :: NTIME
REAL, POINTER, DIMENSION(:) :: XDATA_DTS
REAL, POINTER, DIMENSION(:) :: XDATA_DHUGRD       
!-------------------------------------------------------------------------------
END TYPE DATA_TSZ0_t

TYPE(DATA_TSZ0_t), ALLOCATABLE, TARGET, SAVE :: DATA_TSZ0_MODEL(:)

INTEGER, POINTER :: NTIME=>NULL()
!$OMP THREADPRIVATE(NTIME)
REAL, POINTER, DIMENSION(:)  :: XDATA_DTS => NULL()
!$OMP THREADPRIVATE(XDATA_DTS)
REAL, POINTER, DIMENSION(:)  :: XDATA_DHUGRD => NULL()
!$OMP THREADPRIVATE(XDATA_DHUGRD)

CONTAINS

SUBROUTINE DATA_TSZ0_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
DATA_TSZ0_MODEL(KFROM)%XDATA_DTS=>XDATA_DTS
DATA_TSZ0_MODEL(KFROM)%XDATA_DHUGRD=>XDATA_DHUGRD
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DATA_TSZ0_N:DATA_TSZ0_GOTO_MODEL',0,ZHOOK_HANDLE)
NTIME=>DATA_TSZ0_MODEL(KTO)%NTIME
XDATA_DTS=>DATA_TSZ0_MODEL(KTO)%XDATA_DTS
XDATA_DHUGRD=>DATA_TSZ0_MODEL(KTO)%XDATA_DHUGRD
IF (LHOOK) CALL DR_HOOK('MODD_DATA_TSZ0_N:DATA_TSZ0_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE DATA_TSZ0_GOTO_MODEL

SUBROUTINE DATA_TSZ0_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DATA_TSZ0_N:DATA_TSZ0_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DATA_TSZ0_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(DATA_TSZ0_MODEL(J)%XDATA_DTS)
  NULLIFY(DATA_TSZ0_MODEL(J)%XDATA_DHUGRD)
ENDDO
DATA_TSZ0_MODEL(:)%NTIME=0
IF (LHOOK) CALL DR_HOOK("MODD_DATA_TSZ0_N:DATA_TSZ0_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DATA_TSZ0_ALLOC

SUBROUTINE DATA_TSZ0_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DATA_TSZ0_N:DATA_TSZ0_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DATA_TSZ0_MODEL)) DEALLOCATE(DATA_TSZ0_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DATA_TSZ0_N:DATA_TSZ0_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DATA_TSZ0_DEALLO

END MODULE MODD_DATA_TSZ0_n
