!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ######################
      MODULE MODD_DIAG_WATFLUX_n
!     ######################
!
!!****  *MODD_DIAG_WATFLUX - declaration of diagnostics for WATFLUX scheme
!!
!!    PURPOSE
!!    -------
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	V. Masson   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       01/2004
!!      Modified    01/2006 : sea flux parameterization.
!
!*       0.   DECLARATIONS
!             ------------
!
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE DIAG_WATFLUX_t
!------------------------------------------------------------------------------
!
  REAL    :: XDIAG_TSTEP  ! time step for diagnostics writing
!
  INTEGER :: N2M          ! flag for 2 meters (and 10 meters) quantities
  LOGICAL :: L2M_MIN_ZS   ! flag for 2 meters quantities evaluated on
!                         ! the minimum orographyy of the grid      
  LOGICAL :: LSURF_BUDGET ! flag for surface energy budget
  LOGICAL :: LRAD_BUDGET  ! flag for radiative energy budget      
  LOGICAL :: LCOEF        ! flag for transfer coefficients
  LOGICAL :: LSURF_VARS   ! flag for surface variables
  LOGICAL :: LSURF_BUDGETC       ! flag for surface cumulated energy budget
  LOGICAL :: LRESET_BUDGETC      ! flag for surface cumulated energy budget  
!
!* averaged variables
!
  REAL, POINTER, DIMENSION(:)   :: XRI      ! Bulk-Richardson number           (-)
  REAL, POINTER, DIMENSION(:)   :: XCD      ! drag coefficient for wind        (W/s2)
  REAL, POINTER, DIMENSION(:)   :: XCH      ! drag coefficient for heat        (W/s)
  REAL, POINTER, DIMENSION(:)   :: XCE      ! drag coefficient for vapor       (W/s/K)
  REAL, POINTER, DIMENSION(:)   :: XZ0      ! roughness length for momentum    (m)
  REAL, POINTER, DIMENSION(:)   :: XZ0H     ! roughness length for heat        (m)      
  REAL, POINTER, DIMENSION(:)   :: XRN      ! net radiation at surface         (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XH       ! sensible heat flux               (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLE      ! total latent heat flux           (W/m2) 
  REAL, POINTER, DIMENSION(:)   :: XLEI     ! sublimation latent heat flux     (W/m2) 
  REAL, POINTER, DIMENSION(:)   :: XGFLUX   ! net soil-vegetation flux         (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XT2M     ! air temperature at 2 meters      (K)
  REAL, POINTER, DIMENSION(:)   :: XT2M_MIN ! Minimum air temperature at 2 meters      (K)
  REAL, POINTER, DIMENSION(:)   :: XT2M_MAX ! Maximum air temperature at 2 meters      (K)
  REAL, POINTER, DIMENSION(:)   :: XQ2M     ! air humidity at 2 meters         (kg/kg)
  REAL, POINTER, DIMENSION(:)   :: XHU2M    ! air relative humidity at 2 meters(-)
  REAL, POINTER, DIMENSION(:)   :: XHU2M_MIN! Minimum relative humidity at 2 meters    (-)
  REAL, POINTER, DIMENSION(:)   :: XHU2M_MAX! Maximum relative humidity at 2 meters    (-)
  REAL, POINTER, DIMENSION(:)   :: XQS      ! air humidity at surface          (kg/kg)
  REAL, POINTER, DIMENSION(:)   :: XZON10M  ! zonal wind at 10 meters          (m/s)
  REAL, POINTER, DIMENSION(:)   :: XMER10M  ! meridian wind at 10 meters       (m/s)
  REAL, POINTER, DIMENSION(:)   :: XWIND10M ! wind at 10 meters                (m/s)
  REAL, POINTER, DIMENSION(:)   :: XWIND10M_MAX! Maximum wind at 10 meters     (m/s)
  REAL, POINTER, DIMENSION(:)   :: XLWD     ! downward long wave radiation     (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLWU     ! upward long wave radiation       (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XSWD     ! downward short wave radiation    (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XSWU     ! upward short wave radiation      (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XSWBD    ! downward short wave radiation by spectral band   (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XSWBU    ! upward short wave radiation by spectral band (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XFMU     ! horizontal momentum flux zonal   (kg/ms2)
  REAL, POINTER, DIMENSION(:)   :: XFMV     ! horizontal momentum flux meridian (kg/ms2)
  REAL, POINTER, DIMENSION(:)   :: XDIAG_TS ! water surface temperature (K)
!
!* cumulated averaged variables
!
  REAL, POINTER, DIMENSION(:)   :: XRNC     ! net radiation at surface         (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XHC      ! sensible heat flux               (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XLEC     ! total latent heat flux           (J/m2) 
  REAL, POINTER, DIMENSION(:)   :: XLEIC    ! sublimation latent heat flux     (J/m2) 
  REAL, POINTER, DIMENSION(:)   :: XGFLUXC  ! net soil-vegetation flux         (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XLWDC    ! downward long wave radiation     (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XLWUC    ! upward long wave radiation       (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XSWDC    ! downward short wave radiation    (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XSWUC    ! upward short wave radiation      (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XFMUC    ! horizontal momentum flux zonal    (kg/ms)
  REAL, POINTER, DIMENSION(:)   :: XFMVC    ! horizontal momentum flux meridian (kg/ms)
!  
!------------------------------------------------------------------------------
!

END TYPE DIAG_WATFLUX_t

TYPE(DIAG_WATFLUX_t), ALLOCATABLE, TARGET, SAVE :: DIAG_WATFLUX_MODEL(:)

REAL, POINTER :: XDIAG_TSTEP=>NULL()
!$OMP THREADPRIVATE(XDIAG_TSTEP)
INTEGER, POINTER :: N2M=>NULL()
!$OMP THREADPRIVATE(N2M)
LOGICAL, POINTER :: L2M_MIN_ZS=>NULL()
!$OMP THREADPRIVATE(L2M_MIN_ZS)
LOGICAL, POINTER :: LSURF_BUDGET=>NULL()
!$OMP THREADPRIVATE(LSURF_BUDGET)
LOGICAL, POINTER :: LRAD_BUDGET=>NULL()
!$OMP THREADPRIVATE(LRAD_BUDGET)
LOGICAL, POINTER :: LCOEF=>NULL()
!$OMP THREADPRIVATE(LCOEF)
LOGICAL, POINTER :: LSURF_VARS=>NULL()
!$OMP THREADPRIVATE(LSURF_VARS)
LOGICAL, POINTER :: LSURF_BUDGETC=>NULL()
!$OMP THREADPRIVATE(LSURF_BUDGETC)
LOGICAL, POINTER :: LRESET_BUDGETC=>NULL()
!$OMP THREADPRIVATE(LRESET_BUDGETC)
REAL, POINTER, DIMENSION(:)   :: XRI=>NULL()
!$OMP THREADPRIVATE(XRI)
REAL, POINTER, DIMENSION(:)   :: XCD=>NULL()
!$OMP THREADPRIVATE(XCD)
REAL, POINTER, DIMENSION(:)   :: XCH=>NULL()
!$OMP THREADPRIVATE(XCH)
REAL, POINTER, DIMENSION(:)   :: XCE=>NULL()
!$OMP THREADPRIVATE(XCE)
REAL, POINTER, DIMENSION(:)   :: XZ0=>NULL()
!$OMP THREADPRIVATE(XZ0)
REAL, POINTER, DIMENSION(:)   :: XZ0H=>NULL()
!$OMP THREADPRIVATE(XZ0H)
REAL, POINTER, DIMENSION(:)   :: XRN=>NULL()
!$OMP THREADPRIVATE(XRN)
REAL, POINTER, DIMENSION(:)   :: XH=>NULL()
!$OMP THREADPRIVATE(XH)
REAL, POINTER, DIMENSION(:)   :: XLE=>NULL()
!$OMP THREADPRIVATE(XLE)
REAL, POINTER, DIMENSION(:)   :: XLEI=>NULL()
!$OMP THREADPRIVATE(XLEI)
REAL, POINTER, DIMENSION(:)   :: XGFLUX=>NULL()
!$OMP THREADPRIVATE(XGFLUX)
REAL, POINTER, DIMENSION(:)   :: XT2M=>NULL()
!$OMP THREADPRIVATE(XT2M)
REAL, POINTER, DIMENSION(:)   :: XT2M_MIN=>NULL()
!$OMP THREADPRIVATE(XT2M_MIN)
REAL, POINTER, DIMENSION(:)   :: XT2M_MAX=>NULL()
!$OMP THREADPRIVATE(XT2M_MAX)
REAL, POINTER, DIMENSION(:)   :: XQ2M=>NULL()
!$OMP THREADPRIVATE(XQ2M)
REAL, POINTER, DIMENSION(:)   :: XHU2M=>NULL()
!$OMP THREADPRIVATE(XHU2M)
REAL, POINTER, DIMENSION(:)   :: XHU2M_MIN=>NULL()
!$OMP THREADPRIVATE(XHU2M_MIN)
REAL, POINTER, DIMENSION(:)   :: XHU2M_MAX=>NULL()
!$OMP THREADPRIVATE(XHU2M_MAX)
REAL, POINTER, DIMENSION(:)   :: XQS=>NULL()
!$OMP THREADPRIVATE(XQS)
REAL, POINTER, DIMENSION(:)   :: XZON10M=>NULL()
!$OMP THREADPRIVATE(XZON10M)
REAL, POINTER, DIMENSION(:)   :: XMER10M=>NULL()
!$OMP THREADPRIVATE(XMER10M)
REAL, POINTER, DIMENSION(:)   :: XWIND10M=>NULL()
!$OMP THREADPRIVATE(XWIND10M)
REAL, POINTER, DIMENSION(:)   :: XWIND10M_MAX=>NULL()
!$OMP THREADPRIVATE(XWIND10M_MAX)
REAL, POINTER, DIMENSION(:)   :: XLWD=>NULL()
!$OMP THREADPRIVATE(XLWD)
REAL, POINTER, DIMENSION(:)   :: XLWU=>NULL()
!$OMP THREADPRIVATE(XLWU)
REAL, POINTER, DIMENSION(:)   :: XSWD=>NULL()
!$OMP THREADPRIVATE(XSWD)
REAL, POINTER, DIMENSION(:)   :: XSWU=>NULL()
!$OMP THREADPRIVATE(XSWU)
REAL, POINTER, DIMENSION(:,:) :: XSWBD=>NULL()
!$OMP THREADPRIVATE(XSWBD)
REAL, POINTER, DIMENSION(:,:) :: XSWBU=>NULL()
!$OMP THREADPRIVATE(XSWBU)
REAL, POINTER, DIMENSION(:)   :: XFMU=>NULL()
!$OMP THREADPRIVATE(XFMU)
REAL, POINTER, DIMENSION(:)   :: XFMV=>NULL()
!$OMP THREADPRIVATE(XFMV)
REAL, POINTER, DIMENSION(:)   :: XRNC=>NULL()
!$OMP THREADPRIVATE(XRNC)
REAL, POINTER, DIMENSION(:)   :: XHC=>NULL()
!$OMP THREADPRIVATE(XHC)
REAL, POINTER, DIMENSION(:)   :: XLEC=>NULL()
!$OMP THREADPRIVATE(XLEC)
REAL, POINTER, DIMENSION(:)   :: XLEIC=>NULL()
!$OMP THREADPRIVATE(XLEIC)
REAL, POINTER, DIMENSION(:)   :: XGFLUXC=>NULL()
!$OMP THREADPRIVATE(XGFLUXC)
REAL, POINTER, DIMENSION(:)   :: XLWDC=>NULL()
!$OMP THREADPRIVATE(XLWDC)
REAL, POINTER, DIMENSION(:)   :: XLWUC=>NULL()
!$OMP THREADPRIVATE(XLWUC)
REAL, POINTER, DIMENSION(:)   :: XSWDC=>NULL()
!$OMP THREADPRIVATE(XSWDC)
REAL, POINTER, DIMENSION(:)   :: XSWUC=>NULL()
!$OMP THREADPRIVATE(XSWUC)
REAL, POINTER, DIMENSION(:)   :: XFMUC=>NULL()
!$OMP THREADPRIVATE(XFMUC)
REAL, POINTER, DIMENSION(:)   :: XFMVC=>NULL()
!$OMP THREADPRIVATE(XFMVC)
REAL, POINTER, DIMENSION(:)   :: XDIAG_TS=>NULL()
!$OMP THREADPRIVATE(XDIAG_TS)

CONTAINS

SUBROUTINE DIAG_WATFLUX_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
DIAG_WATFLUX_MODEL(KFROM)%XRI=>XRI
DIAG_WATFLUX_MODEL(KFROM)%XCD=>XCD
DIAG_WATFLUX_MODEL(KFROM)%XCH=>XCH
DIAG_WATFLUX_MODEL(KFROM)%XCE=>XCE
DIAG_WATFLUX_MODEL(KFROM)%XZ0=>XZ0
DIAG_WATFLUX_MODEL(KFROM)%XZ0H=>XZ0H
DIAG_WATFLUX_MODEL(KFROM)%XRN=>XRN
DIAG_WATFLUX_MODEL(KFROM)%XH=>XH
DIAG_WATFLUX_MODEL(KFROM)%XLE=>XLE
DIAG_WATFLUX_MODEL(KFROM)%XLEI=>XLEI
DIAG_WATFLUX_MODEL(KFROM)%XGFLUX=>XGFLUX
DIAG_WATFLUX_MODEL(KFROM)%XT2M=>XT2M
DIAG_WATFLUX_MODEL(KFROM)%XT2M_MIN=>XT2M_MIN
DIAG_WATFLUX_MODEL(KFROM)%XT2M_MAX=>XT2M_MAX
DIAG_WATFLUX_MODEL(KFROM)%XQ2M=>XQ2M
DIAG_WATFLUX_MODEL(KFROM)%XHU2M=>XHU2M
DIAG_WATFLUX_MODEL(KFROM)%XHU2M_MIN=>XHU2M_MIN
DIAG_WATFLUX_MODEL(KFROM)%XHU2M_MAX=>XHU2M_MAX
DIAG_WATFLUX_MODEL(KFROM)%XQS=>XQS
DIAG_WATFLUX_MODEL(KFROM)%XZON10M=>XZON10M
DIAG_WATFLUX_MODEL(KFROM)%XMER10M=>XMER10M
DIAG_WATFLUX_MODEL(KFROM)%XWIND10M=>XWIND10M
DIAG_WATFLUX_MODEL(KFROM)%XWIND10M_MAX=>XWIND10M_MAX
DIAG_WATFLUX_MODEL(KFROM)%XLWD=>XLWD
DIAG_WATFLUX_MODEL(KFROM)%XLWU=>XLWU
DIAG_WATFLUX_MODEL(KFROM)%XSWD=>XSWD
DIAG_WATFLUX_MODEL(KFROM)%XSWU=>XSWU
DIAG_WATFLUX_MODEL(KFROM)%XSWBD=>XSWBD
DIAG_WATFLUX_MODEL(KFROM)%XSWBU=>XSWBU
DIAG_WATFLUX_MODEL(KFROM)%XFMU=>XFMU
DIAG_WATFLUX_MODEL(KFROM)%XFMV=>XFMV
DIAG_WATFLUX_MODEL(KFROM)%XRNC=>XRNC
DIAG_WATFLUX_MODEL(KFROM)%XHC=>XHC
DIAG_WATFLUX_MODEL(KFROM)%XLEC=>XLEC
DIAG_WATFLUX_MODEL(KFROM)%XLEIC=>XLEIC
DIAG_WATFLUX_MODEL(KFROM)%XGFLUXC=>XGFLUXC
DIAG_WATFLUX_MODEL(KFROM)%XLWDC=>XLWDC
DIAG_WATFLUX_MODEL(KFROM)%XLWUC=>XLWUC
DIAG_WATFLUX_MODEL(KFROM)%XSWDC=>XSWDC
DIAG_WATFLUX_MODEL(KFROM)%XSWUC=>XSWUC
DIAG_WATFLUX_MODEL(KFROM)%XFMUC=>XFMUC
DIAG_WATFLUX_MODEL(KFROM)%XFMVC=>XFMVC
DIAG_WATFLUX_MODEL(KFROM)%XDIAG_TS=>XDIAG_TS
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_WATFLUX_N:DIAG_WATFLUX_GOTO_MODEL',0,ZHOOK_HANDLE)
XDIAG_TSTEP=>DIAG_WATFLUX_MODEL(KTO)%XDIAG_TSTEP
N2M=>DIAG_WATFLUX_MODEL(KTO)%N2M
L2M_MIN_ZS=>DIAG_WATFLUX_MODEL(KTO)%L2M_MIN_ZS
LSURF_BUDGET=>DIAG_WATFLUX_MODEL(KTO)%LSURF_BUDGET
LRAD_BUDGET=>DIAG_WATFLUX_MODEL(KTO)%LRAD_BUDGET
LCOEF=>DIAG_WATFLUX_MODEL(KTO)%LCOEF
LSURF_VARS=>DIAG_WATFLUX_MODEL(KTO)%LSURF_VARS
LSURF_BUDGETC=>DIAG_WATFLUX_MODEL(KTO)%LSURF_BUDGETC
LRESET_BUDGETC=>DIAG_WATFLUX_MODEL(KTO)%LRESET_BUDGETC
XRI=>DIAG_WATFLUX_MODEL(KTO)%XRI
XCD=>DIAG_WATFLUX_MODEL(KTO)%XCD
XCH=>DIAG_WATFLUX_MODEL(KTO)%XCH
XCE=>DIAG_WATFLUX_MODEL(KTO)%XCE
XZ0=>DIAG_WATFLUX_MODEL(KTO)%XZ0
XZ0H=>DIAG_WATFLUX_MODEL(KTO)%XZ0H
XRN=>DIAG_WATFLUX_MODEL(KTO)%XRN
XH=>DIAG_WATFLUX_MODEL(KTO)%XH
XLE=>DIAG_WATFLUX_MODEL(KTO)%XLE
XLEI=>DIAG_WATFLUX_MODEL(KTO)%XLEI
XGFLUX=>DIAG_WATFLUX_MODEL(KTO)%XGFLUX
XT2M=>DIAG_WATFLUX_MODEL(KTO)%XT2M
XT2M_MIN=>DIAG_WATFLUX_MODEL(KTO)%XT2M_MIN
XT2M_MAX=>DIAG_WATFLUX_MODEL(KTO)%XT2M_MAX
XQ2M=>DIAG_WATFLUX_MODEL(KTO)%XQ2M
XHU2M=>DIAG_WATFLUX_MODEL(KTO)%XHU2M
XHU2M_MIN=>DIAG_WATFLUX_MODEL(KTO)%XHU2M_MIN
XHU2M_MAX=>DIAG_WATFLUX_MODEL(KTO)%XHU2M_MAX
XQS=>DIAG_WATFLUX_MODEL(KTO)%XQS
XZON10M=>DIAG_WATFLUX_MODEL(KTO)%XZON10M
XMER10M=>DIAG_WATFLUX_MODEL(KTO)%XMER10M
XWIND10M=>DIAG_WATFLUX_MODEL(KTO)%XWIND10M
XWIND10M_MAX=>DIAG_WATFLUX_MODEL(KTO)%XWIND10M_MAX
XLWD=>DIAG_WATFLUX_MODEL(KTO)%XLWD
XLWU=>DIAG_WATFLUX_MODEL(KTO)%XLWU
XSWD=>DIAG_WATFLUX_MODEL(KTO)%XSWD
XSWU=>DIAG_WATFLUX_MODEL(KTO)%XSWU
XSWBD=>DIAG_WATFLUX_MODEL(KTO)%XSWBD
XSWBU=>DIAG_WATFLUX_MODEL(KTO)%XSWBU
XFMU=>DIAG_WATFLUX_MODEL(KTO)%XFMU
XFMV=>DIAG_WATFLUX_MODEL(KTO)%XFMV
XRNC=>DIAG_WATFLUX_MODEL(KTO)%XRNC
XHC=>DIAG_WATFLUX_MODEL(KTO)%XHC
XLEC=>DIAG_WATFLUX_MODEL(KTO)%XLEC
XLEIC=>DIAG_WATFLUX_MODEL(KTO)%XLEIC
XGFLUXC=>DIAG_WATFLUX_MODEL(KTO)%XGFLUXC
XLWDC=>DIAG_WATFLUX_MODEL(KTO)%XLWDC
XLWUC=>DIAG_WATFLUX_MODEL(KTO)%XLWUC
XSWDC=>DIAG_WATFLUX_MODEL(KTO)%XSWDC
XSWUC=>DIAG_WATFLUX_MODEL(KTO)%XSWUC
XFMUC=>DIAG_WATFLUX_MODEL(KTO)%XFMUC
XFMVC=>DIAG_WATFLUX_MODEL(KTO)%XFMVC
XDIAG_TS=>DIAG_WATFLUX_MODEL(KTO)%XDIAG_TS
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_WATFLUX_N:DIAG_WATFLUX_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE DIAG_WATFLUX_GOTO_MODEL

SUBROUTINE DIAG_WATFLUX_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_WATFLUX_N:DIAG_WATFLUX_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DIAG_WATFLUX_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XRI)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XCD)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XCH)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XCE)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XZ0)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XZ0H)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XRN)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XH)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XLE)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XLEI)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XGFLUX)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XT2M)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XT2M_MIN)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XT2M_MAX)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XQ2M)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XHU2M)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XHU2M_MIN)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XHU2M_MAX)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XQS)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XZON10M)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XMER10M)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XWIND10M)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XWIND10M_MAX)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XLWD)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XLWU)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XSWD)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XSWU)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XSWBD)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XSWBU)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XFMU)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XFMV)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XDIAG_TS)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XRNC)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XHC)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XLEC)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XLEIC)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XGFLUXC)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XLWDC)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XLWUC)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XSWDC)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XSWUC)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XFMUC)
  NULLIFY(DIAG_WATFLUX_MODEL(J)%XFMVC)
ENDDO
DIAG_WATFLUX_MODEL(:)%XDIAG_TSTEP=0.
DIAG_WATFLUX_MODEL(:)%N2M=0
DIAG_WATFLUX_MODEL(:)%L2M_MIN_ZS=.FALSE.
DIAG_WATFLUX_MODEL(:)%LSURF_BUDGET=.FALSE.
DIAG_WATFLUX_MODEL(:)%LRAD_BUDGET=.FALSE.
DIAG_WATFLUX_MODEL(:)%LCOEF=.FALSE.
DIAG_WATFLUX_MODEL(:)%LSURF_VARS=.FALSE.
DIAG_WATFLUX_MODEL(:)%LSURF_BUDGETC=.FALSE.
DIAG_WATFLUX_MODEL(:)%LRESET_BUDGETC=.FALSE.
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_WATFLUX_N:DIAG_WATFLUX_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_WATFLUX_ALLOC

SUBROUTINE DIAG_WATFLUX_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_WATFLUX_N:DIAG_WATFLUX_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DIAG_WATFLUX_MODEL)) DEALLOCATE(DIAG_WATFLUX_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_WATFLUX_N:DIAG_WATFLUX_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_WATFLUX_DEALLO

END MODULE MODD_DIAG_WATFLUX_n
