!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ####################
MODULE MODD_FLAKE_n
!     ####################
!
!!****  *MODD_FLAKE_n - declaration of surface parameters for the FLake model 
!!                      for inland water surfaces
!!
!!    PURPOSE
!!    -------
!     Declaration of surface parameters
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	V. Masson  *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       01/2004
!
!*       0.   DECLARATIONS
!             ------------
!
USE MODD_TYPE_DATE_SURF
!

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
TYPE FLAKE_t 
!
! General surface: 
!
  REAL, POINTER, DIMENSION(:) :: XZS       ! orography                     (m)
  REAL, POINTER, DIMENSION(:) :: XZ0       ! roughness length              (m)
  REAL, POINTER, DIMENSION(:) :: XUSTAR    ! air friction velocity         (m/s)
  REAL, POINTER, DIMENSION(:,:) :: XCOVER  ! fraction of each ecosystem    (-)
  LOGICAL, POINTER, DIMENSION(:):: LCOVER  ! GCOVER(i)=T --> ith cover field is not 0.
  LOGICAL                       :: LSBL    ! T: SBL scheme within the Surface Boundary Layer
!                                          ! F: no atmospheric layers below forcing level  
!
! Inland water:
  REAL, POINTER, DIMENSION(:) :: XEMIS  ! water surface emissivity (NOT USED BY FLAKE)
! FLake parameters
  REAL, POINTER, DIMENSION(:) :: XWATER_DEPTH  ! Lake depth (m)
  REAL, POINTER, DIMENSION(:) :: XWATER_FETCH  ! Lake fetch (m)
  REAL, POINTER, DIMENSION(:) :: XT_BS         ! Temperature at the outer edge of the thermally 
                                               !       active layer of the bottom sediments [K]
  REAL, POINTER, DIMENSION(:) :: XDEPTH_BS     ! Depth of the thermally active layer of the
                                               !       bottom sediments [m]
  REAL, POINTER, DIMENSION(:) :: XCORIO        ! The Coriolis parameter [s^{-1}]
  REAL, POINTER, DIMENSION(:) :: XDIR_ALB      ! Water surface direct albedo
  REAL, POINTER, DIMENSION(:) :: XSCA_ALB      ! Water surface diffuse albedo
  REAL, POINTER, DIMENSION(:) :: XICE_ALB      ! Ice surface albedo (for ESM coupling)
  REAL, POINTER, DIMENSION(:) :: XSNOW_ALB     ! Snow surface albedo
  REAL, POINTER, DIMENSION(:) :: XEXTCOEF_WATER ! Extinction coefficient for the water [m^{-1}]
  REAL, POINTER, DIMENSION(:) :: XEXTCOEF_ICE   ! Extinction coefficient for the ice [m^{-1}]
  REAL, POINTER, DIMENSION(:) :: XEXTCOEF_SNOW  ! Extinction coefficient for the snow [m^{-1}] 
! Flake variables
  REAL, POINTER, DIMENSION(:) :: XT_SNOW       ! Temperature at the air-snow interface [K]    
  REAL, POINTER, DIMENSION(:) :: XT_ICE        ! Temperature at the snow-ice or air-ice 
                                               !        interface [K]
  REAL, POINTER, DIMENSION(:) :: XT_MNW        ! Mean temperature of the water column [K]
  REAL, POINTER, DIMENSION(:) :: XT_WML        ! Mixed-layer temperature [K]
  REAL, POINTER, DIMENSION(:) :: XT_BOT        ! Temperature at the water-bottom sediment 
                                               !        interface [K]
  REAL, POINTER, DIMENSION(:) :: XT_B1         ! Temperature at the bottom of the upper 
                                               !        layer of the sediments [K]
  REAL, POINTER, DIMENSION(:) :: XCT           ! Shape factor (thermocline)
  REAL, POINTER, DIMENSION(:) :: XH_SNOW       ! Snow thickness [m]
  REAL, POINTER, DIMENSION(:) :: XH_ICE        ! Ice thickness [m]
  REAL, POINTER, DIMENSION(:) :: XH_ML         ! Thickness of the mixed-layer [m]
  REAL, POINTER, DIMENSION(:) :: XH_B1         ! Thickness of the upper layer of bottom sediments [m]                                    
!
  REAL, POINTER, DIMENSION(:) :: XTS  ! surface temperature  (K)
                                      ! (water or ice or snow)

! Date:
!
  TYPE (DATE_TIME)                  :: TTIME         ! current date and time
!
! Time-step:
!
  REAL                              :: XTSTEP        ! time step
!
  REAL                              :: XOUT_TSTEP    ! output writing time step
!
! FLake switches
!
  LOGICAL            :: LSEDIMENTS  ! flag to use or not the bottom sediments
  CHARACTER(LEN=3)   :: CSNOW_FLK   ! FLake snow scheme
  CHARACTER(LEN=5)   :: CFLK_FLUX   ! Type of flux computation
  CHARACTER(LEN=4)   :: CFLK_ALB    ! Type of albedo
!
! ECUME switches for FLake
  LOGICAL                           :: LPRECIP     ! flag for precip correction
  LOGICAL                           :: LPWEBB      ! flag for heat flux correction
  REAL                              :: XICHCE      ! CE coef calculation for ECUME
!
END TYPE FLAKE_t

TYPE(FLAKE_t), ALLOCATABLE, TARGET, SAVE :: FLAKE_MODEL(:)

REAL, POINTER, DIMENSION(:) :: XZS=>NULL()
!$OMP THREADPRIVATE(XZS)
REAL, POINTER, DIMENSION(:) :: XZ0=>NULL()
!$OMP THREADPRIVATE(XZ0)
REAL, POINTER, DIMENSION(:) :: XUSTAR=>NULL()
!$OMP THREADPRIVATE(XUSTAR)
REAL, POINTER, DIMENSION(:,:) :: XCOVER=>NULL()
!$OMP THREADPRIVATE(XCOVER)
LOGICAL, POINTER, DIMENSION(:):: LCOVER=>NULL()
!$OMP THREADPRIVATE(LCOVER)
LOGICAL, POINTER :: LSBL=>NULL()
!$OMP THREADPRIVATE(LSBL)
REAL, POINTER, DIMENSION(:) :: XEMIS=>NULL()
!$OMP THREADPRIVATE(XEMIS)
REAL, POINTER, DIMENSION(:) :: XWATER_DEPTH=>NULL()
!$OMP THREADPRIVATE(XWATER_DEPTH)
REAL, POINTER, DIMENSION(:) :: XWATER_FETCH=>NULL()
!$OMP THREADPRIVATE(XWATER_FETCH)
REAL, POINTER, DIMENSION(:) :: XT_BS=>NULL()
!$OMP THREADPRIVATE(XT_BS)
REAL, POINTER, DIMENSION(:) :: XDEPTH_BS=>NULL()
!$OMP THREADPRIVATE(XDEPTH_BS)
REAL, POINTER, DIMENSION(:) :: XCORIO=>NULL()
!$OMP THREADPRIVATE(XCORIO)
REAL, POINTER, DIMENSION(:) :: XDIR_ALB=>NULL()
!$OMP THREADPRIVATE(XDIR_ALB)
REAL, POINTER, DIMENSION(:) :: XSCA_ALB=>NULL()
!$OMP THREADPRIVATE(XSCA_ALB)
REAL, POINTER, DIMENSION(:) :: XICE_ALB=>NULL()
!$OMP THREADPRIVATE(XICE_ALB)
REAL, POINTER, DIMENSION(:) :: XSNOW_ALB=>NULL()
!$OMP THREADPRIVATE(XSNOW_ALB)
REAL, POINTER, DIMENSION(:) :: XEXTCOEF_WATER=>NULL()
!$OMP THREADPRIVATE(XEXTCOEF_WATER)
REAL, POINTER, DIMENSION(:) :: XEXTCOEF_ICE=>NULL()
!$OMP THREADPRIVATE(XEXTCOEF_ICE)
REAL, POINTER, DIMENSION(:) :: XEXTCOEF_SNOW=>NULL()
!$OMP THREADPRIVATE(XEXTCOEF_SNOW)
REAL, POINTER, DIMENSION(:) :: XT_SNOW=>NULL()
!$OMP THREADPRIVATE(XT_SNOW)
REAL, POINTER, DIMENSION(:) :: XT_ICE=>NULL()
!$OMP THREADPRIVATE(XT_ICE)
REAL, POINTER, DIMENSION(:) :: XT_MNW=>NULL()
!$OMP THREADPRIVATE(XT_MNW)
REAL, POINTER, DIMENSION(:) :: XT_WML=>NULL()
!$OMP THREADPRIVATE(XT_WML)
REAL, POINTER, DIMENSION(:) :: XT_BOT=>NULL()
!$OMP THREADPRIVATE(XT_BOT)
REAL, POINTER, DIMENSION(:) :: XT_B1=>NULL()
!$OMP THREADPRIVATE(XT_B1)
REAL, POINTER, DIMENSION(:) :: XCT=>NULL()
!$OMP THREADPRIVATE(XCT)
REAL, POINTER, DIMENSION(:) :: XH_SNOW=>NULL()
!$OMP THREADPRIVATE(XH_SNOW)
REAL, POINTER, DIMENSION(:) :: XH_ICE=>NULL()
!$OMP THREADPRIVATE(XH_ICE)
REAL, POINTER, DIMENSION(:) :: XH_ML=>NULL()
!$OMP THREADPRIVATE(XH_ML)
REAL, POINTER, DIMENSION(:) :: XH_B1=>NULL()
!$OMP THREADPRIVATE(XH_B1)
REAL, POINTER, DIMENSION(:) :: XTS=>NULL()
!$OMP THREADPRIVATE(XTS)
TYPE (DATE_TIME), POINTER :: TTIME=>NULL()
!$OMP THREADPRIVATE(TTIME)
REAL, POINTER :: XTSTEP=>NULL()
!$OMP THREADPRIVATE(XTSTEP)
REAL, POINTER :: XOUT_TSTEP=>NULL()
!$OMP THREADPRIVATE(XOUT_TSTEP)
!
LOGICAL, POINTER            :: LSEDIMENTS=>NULL()  
!$OMP THREADPRIVATE(LSEDIMENTS)
 CHARACTER(LEN=3),POINTER    :: CSNOW_FLK=>NULL()
!$OMP THREADPRIVATE(CSNOW_FLK)
 CHARACTER(LEN=4),POINTER    :: CFLK_ALB=>NULL()
!$OMP THREADPRIVATE(CFLK_ALB)
 CHARACTER(LEN=5),POINTER    :: CFLK_FLUX=>NULL()
!$OMP THREADPRIVATE(CFLK_FLUX)
!
REAL, POINTER :: XICHCE=>NULL()
!$OMP THREADPRIVATE(XICHCE)
LOGICAL, POINTER :: LPRECIP=>NULL()
!$OMP THREADPRIVATE(LPRECIP)
LOGICAL, POINTER :: LPWEBB=>NULL()
!$OMP THREADPRIVATE(LPWEBB)

CONTAINS

SUBROUTINE FLAKE_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
FLAKE_MODEL(KFROM)%XZS=>XZS
FLAKE_MODEL(KFROM)%XZ0=>XZ0
FLAKE_MODEL(KFROM)%XUSTAR=>XUSTAR
FLAKE_MODEL(KFROM)%XCOVER=>XCOVER
FLAKE_MODEL(KFROM)%LCOVER=>LCOVER
FLAKE_MODEL(KFROM)%XEMIS=>  XEMIS
FLAKE_MODEL(KFROM)%XWATER_DEPTH=>  XWATER_DEPTH
FLAKE_MODEL(KFROM)%XWATER_FETCH=>  XWATER_FETCH
FLAKE_MODEL(KFROM)%XT_BS=>         XT_BS
FLAKE_MODEL(KFROM)%XDEPTH_BS=>     XDEPTH_BS
FLAKE_MODEL(KFROM)%XCORIO=>        XCORIO
FLAKE_MODEL(KFROM)%XDIR_ALB=>      XDIR_ALB
FLAKE_MODEL(KFROM)%XSCA_ALB=>      XSCA_ALB
FLAKE_MODEL(KFROM)%XICE_ALB=>      XICE_ALB
FLAKE_MODEL(KFROM)%XSNOW_ALB=>     XSNOW_ALB
FLAKE_MODEL(KFROM)%XEXTCOEF_WATER=>XEXTCOEF_WATER
FLAKE_MODEL(KFROM)%XEXTCOEF_ICE=>  XEXTCOEF_ICE
FLAKE_MODEL(KFROM)%XEXTCOEF_SNOW=> XEXTCOEF_SNOW
FLAKE_MODEL(KFROM)%XT_SNOW=>       XT_SNOW
FLAKE_MODEL(KFROM)%XT_ICE=>        XT_ICE
FLAKE_MODEL(KFROM)%XT_MNW=>        XT_MNW
FLAKE_MODEL(KFROM)%XT_WML=>        XT_WML
FLAKE_MODEL(KFROM)%XT_BOT=>        XT_BOT
FLAKE_MODEL(KFROM)%XT_B1=>         XT_B1
FLAKE_MODEL(KFROM)%XCT=>           XCT
FLAKE_MODEL(KFROM)%XH_SNOW=>       XH_SNOW
FLAKE_MODEL(KFROM)%XH_ICE=>        XH_ICE
FLAKE_MODEL(KFROM)%XH_ML=>         XH_ML
FLAKE_MODEL(KFROM)%XH_B1=>         XH_B1            
FLAKE_MODEL(KFROM)%XTS=>XTS
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_FLAKE_N:FLAKE_GOTO_MODEL',0,ZHOOK_HANDLE)
XZS=>FLAKE_MODEL(KTO)%XZS
XZ0=>FLAKE_MODEL(KTO)%XZ0
XUSTAR=>FLAKE_MODEL(KTO)%XUSTAR
XCOVER=>FLAKE_MODEL(KTO)%XCOVER
LCOVER=>FLAKE_MODEL(KTO)%LCOVER
LSBL=>FLAKE_MODEL(KTO)%LSBL
XEMIS  =>FLAKE_MODEL(KTO)%XEMIS
XWATER_DEPTH  =>FLAKE_MODEL(KTO)%XWATER_DEPTH
XWATER_FETCH  =>FLAKE_MODEL(KTO)%XWATER_FETCH
XT_BS         =>FLAKE_MODEL(KTO)%XT_BS
XDEPTH_BS     =>FLAKE_MODEL(KTO)%XDEPTH_BS
XCORIO        =>FLAKE_MODEL(KTO)%XCORIO
XDIR_ALB      =>FLAKE_MODEL(KTO)%XDIR_ALB
XSCA_ALB      =>FLAKE_MODEL(KTO)%XSCA_ALB
XICE_ALB      =>FLAKE_MODEL(KTO)%XICE_ALB
XSNOW_ALB     =>FLAKE_MODEL(KTO)%XSNOW_ALB
XEXTCOEF_WATER =>FLAKE_MODEL(KTO)%XEXTCOEF_WATER
XEXTCOEF_ICE  =>FLAKE_MODEL(KTO)%XEXTCOEF_ICE
XEXTCOEF_SNOW =>FLAKE_MODEL(KTO)%XEXTCOEF_SNOW
XT_SNOW       =>FLAKE_MODEL(KTO)%XT_SNOW
XT_ICE        =>FLAKE_MODEL(KTO)%XT_ICE
XT_MNW        =>FLAKE_MODEL(KTO)%XT_MNW
XT_WML        =>FLAKE_MODEL(KTO)%XT_WML
XT_BOT        =>FLAKE_MODEL(KTO)%XT_BOT
XT_B1         =>FLAKE_MODEL(KTO)%XT_B1
XCT           =>FLAKE_MODEL(KTO)%XCT
XH_SNOW       =>FLAKE_MODEL(KTO)%XH_SNOW
XH_ICE        =>FLAKE_MODEL(KTO)%XH_ICE
XH_ML         =>FLAKE_MODEL(KTO)%XH_ML
XH_B1         =>FLAKE_MODEL(KTO)%XH_B1        
XTS=>FLAKE_MODEL(KTO)%XTS
TTIME=>FLAKE_MODEL(KTO)%TTIME
XTSTEP=>FLAKE_MODEL(KTO)%XTSTEP
XOUT_TSTEP=>FLAKE_MODEL(KTO)%XOUT_TSTEP
!
LSEDIMENTS=>FLAKE_MODEL(KTO)%LSEDIMENTS  
CSNOW_FLK=>FLAKE_MODEL(KTO)%CSNOW_FLK
CFLK_ALB=>FLAKE_MODEL(KTO)%CFLK_ALB
CFLK_FLUX=>FLAKE_MODEL(KTO)%CFLK_FLUX
!
XICHCE=>FLAKE_MODEL(KTO)%XICHCE
LPRECIP=>FLAKE_MODEL(KTO)%LPRECIP  
LPWEBB=>FLAKE_MODEL(KTO)%LPWEBB  
!
IF (LHOOK) CALL DR_HOOK('MODD_FLAKE_N:FLAKE_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE FLAKE_GOTO_MODEL

SUBROUTINE FLAKE_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_FLAKE_N:FLAKE_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(FLAKE_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(FLAKE_MODEL(J)%XZS)
  NULLIFY(FLAKE_MODEL(J)%XZ0)
  NULLIFY(FLAKE_MODEL(J)%XUSTAR)
  NULLIFY(FLAKE_MODEL(J)%XCOVER)
  NULLIFY(FLAKE_MODEL(J)%LCOVER)
  NULLIFY(FLAKE_MODEL(J)%XEMIS)
  NULLIFY(FLAKE_MODEL(J)%XWATER_DEPTH)
  NULLIFY(FLAKE_MODEL(J)%XWATER_FETCH)
  NULLIFY(FLAKE_MODEL(J)%XT_BS)
  NULLIFY(FLAKE_MODEL(J)%XDEPTH_BS)
  NULLIFY(FLAKE_MODEL(J)%XCORIO)
  NULLIFY(FLAKE_MODEL(J)%XDIR_ALB)
  NULLIFY(FLAKE_MODEL(J)%XSCA_ALB)
  NULLIFY(FLAKE_MODEL(J)%XICE_ALB)
  NULLIFY(FLAKE_MODEL(J)%XSNOW_ALB)
  NULLIFY(FLAKE_MODEL(J)%XEXTCOEF_WATER)
  NULLIFY(FLAKE_MODEL(J)%XEXTCOEF_ICE)
  NULLIFY(FLAKE_MODEL(J)%XEXTCOEF_SNOW)
  NULLIFY(FLAKE_MODEL(J)%XT_SNOW)
  NULLIFY(FLAKE_MODEL(J)%XT_ICE)
  NULLIFY(FLAKE_MODEL(J)%XT_MNW)
  NULLIFY(FLAKE_MODEL(J)%XT_WML)
  NULLIFY(FLAKE_MODEL(J)%XT_BOT)
  NULLIFY(FLAKE_MODEL(J)%XT_B1)
  NULLIFY(FLAKE_MODEL(J)%XCT)
  NULLIFY(FLAKE_MODEL(J)%XH_SNOW)
  NULLIFY(FLAKE_MODEL(J)%XH_ICE)
  NULLIFY(FLAKE_MODEL(J)%XH_ML)
  NULLIFY(FLAKE_MODEL(J)%XH_B1)
  NULLIFY(FLAKE_MODEL(J)%XTS)
ENDDO
FLAKE_MODEL(:)%LSBL=.FALSE.
FLAKE_MODEL(:)%XTSTEP=0.
FLAKE_MODEL(:)%XOUT_TSTEP=0.
FLAKE_MODEL(:)%LSEDIMENTS=.FALSE.
FLAKE_MODEL(:)%CSNOW_FLK='   '
FLAKE_MODEL(:)%CFLK_ALB='    '
FLAKE_MODEL(:)%CFLK_FLUX='     '
FLAKE_MODEL(:)%XICHCE=0.
FLAKE_MODEL(:)%LPRECIP=.FALSE.
FLAKE_MODEL(:)%LPWEBB=.FALSE.
IF (LHOOK) CALL DR_HOOK("MODD_FLAKE_N:FLAKE_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE FLAKE_ALLOC

SUBROUTINE FLAKE_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_FLAKE_N:FLAKE_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(FLAKE_MODEL)) DEALLOCATE(FLAKE_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_FLAKE_N:FLAKE_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE FLAKE_DEALLO

END MODULE MODD_FLAKE_n
