!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ################
      MODULE MODD_SEAFLUX_SBL_n
!     ################
!
!!****  *MODD_SEAFLUX_SBL_n - declaration of surface parameters for SBL scheme
!!
!!    PURPOSE
!!    -------
!     Declaration of surface parameters
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	V. Masson   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       07/2006
!
!*       0.   DECLARATIONS
!             ------------
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE SEAFLUX_SBL_t
!
  INTEGER                       :: NLVL ! number      of levels in SBL
  REAL, POINTER, DIMENSION(:,:) :: XZ   ! height of middle of each level grid   (m)
  REAL, POINTER, DIMENSION(:,:) :: XU   ! wind        at each level in SBL      (m/s)
  REAL, POINTER, DIMENSION(:,:) :: XT   ! temperature at each level in SBL      (m/s)
  REAL, POINTER, DIMENSION(:,:) :: XQ   ! humidity    at each level in SBL      (kg/m3)
  REAL, POINTER, DIMENSION(:,:) :: XTKE ! Tke         at each level in SBL      (m2/s2)
  REAL, POINTER, DIMENSION(:)   :: XLMO ! Monin-Obhukov length                  (m)
  REAL, POINTER, DIMENSION(:,:) :: XP   ! pressure    at each level in SBL      (kg/m3)
!
  REAL, POINTER, DIMENSION(:,:) :: XDZ  ! depth       of each level in SBL      (m)
  REAL, POINTER, DIMENSION(:,:) :: XZF  ! height of bottom of each level grid   (m)
  REAL, POINTER, DIMENSION(:,:) :: XDZF ! depth between  each level in SBL      (m)
!
END TYPE SEAFLUX_SBL_t

TYPE(SEAFLUX_SBL_t), ALLOCATABLE, TARGET, SAVE :: SEAFLUX_SBL_MODEL(:)

INTEGER, POINTER :: NLVL=>NULL()
!$OMP THREADPRIVATE(NLVL)
REAL, POINTER, DIMENSION(:,:) :: XZ=>NULL()
!$OMP THREADPRIVATE(XZ)
REAL, POINTER, DIMENSION(:,:) :: XU=>NULL()
!$OMP THREADPRIVATE(XU)
REAL, POINTER, DIMENSION(:,:) :: XT=>NULL()
!$OMP THREADPRIVATE(XT)
REAL, POINTER, DIMENSION(:,:) :: XQ=>NULL()
!$OMP THREADPRIVATE(XQ)
REAL, POINTER, DIMENSION(:,:) :: XTKE=>NULL()
!$OMP THREADPRIVATE(XTKE)
REAL, POINTER, DIMENSION(:)   :: XLMO=>NULL()
!$OMP THREADPRIVATE(XLMO)
REAL, POINTER, DIMENSION(:,:) :: XP=>NULL()
!$OMP THREADPRIVATE(XP)
REAL, POINTER, DIMENSION(:,:) :: XDZ=>NULL()
!$OMP THREADPRIVATE(XDZ)
REAL, POINTER, DIMENSION(:,:) :: XZF=>NULL()
!$OMP THREADPRIVATE(XZF)
REAL, POINTER, DIMENSION(:,:) :: XDZF=>NULL()
!$OMP THREADPRIVATE(XDZF)

CONTAINS

SUBROUTINE SEAFLUX_SBL_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
SEAFLUX_SBL_MODEL(KFROM)%XZ=>XZ
SEAFLUX_SBL_MODEL(KFROM)%XU=>XU
SEAFLUX_SBL_MODEL(KFROM)%XT=>XT
SEAFLUX_SBL_MODEL(KFROM)%XQ=>XQ
SEAFLUX_SBL_MODEL(KFROM)%XTKE=>XTKE
SEAFLUX_SBL_MODEL(KFROM)%XLMO=>XLMO
SEAFLUX_SBL_MODEL(KFROM)%XP=>XP
SEAFLUX_SBL_MODEL(KFROM)%XDZ=>XDZ
SEAFLUX_SBL_MODEL(KFROM)%XZF=>XZF
SEAFLUX_SBL_MODEL(KFROM)%XDZF=>XDZF
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_SEAFLUX_SBL_N:SEAFLUX_SBL_GOTO_MODEL',0,ZHOOK_HANDLE)
NLVL=>SEAFLUX_SBL_MODEL(KTO)%NLVL
XZ=>SEAFLUX_SBL_MODEL(KTO)%XZ
XU=>SEAFLUX_SBL_MODEL(KTO)%XU
XT=>SEAFLUX_SBL_MODEL(KTO)%XT
XQ=>SEAFLUX_SBL_MODEL(KTO)%XQ
XTKE=>SEAFLUX_SBL_MODEL(KTO)%XTKE
XLMO=>SEAFLUX_SBL_MODEL(KTO)%XLMO
XP=>SEAFLUX_SBL_MODEL(KTO)%XP
XDZ=>SEAFLUX_SBL_MODEL(KTO)%XDZ
XZF=>SEAFLUX_SBL_MODEL(KTO)%XZF
XDZF=>SEAFLUX_SBL_MODEL(KTO)%XDZF
IF (LHOOK) CALL DR_HOOK('MODD_SEAFLUX_SBL_N:SEAFLUX_SBL_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE SEAFLUX_SBL_GOTO_MODEL

SUBROUTINE SEAFLUX_SBL_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_SEAFLUX_SBL_N:SEAFLUX_SBL_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(SEAFLUX_SBL_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(SEAFLUX_SBL_MODEL(J)%XZ)
  NULLIFY(SEAFLUX_SBL_MODEL(J)%XU)
  NULLIFY(SEAFLUX_SBL_MODEL(J)%XT)
  NULLIFY(SEAFLUX_SBL_MODEL(J)%XQ)
  NULLIFY(SEAFLUX_SBL_MODEL(J)%XTKE)
  NULLIFY(SEAFLUX_SBL_MODEL(J)%XLMO)
  NULLIFY(SEAFLUX_SBL_MODEL(J)%XP)
  NULLIFY(SEAFLUX_SBL_MODEL(J)%XDZ)
  NULLIFY(SEAFLUX_SBL_MODEL(J)%XZF)
  NULLIFY(SEAFLUX_SBL_MODEL(J)%XDZF)
ENDDO
SEAFLUX_SBL_MODEL(:)%NLVL=0
IF (LHOOK) CALL DR_HOOK("MODD_SEAFLUX_SBL_N:SEAFLUX_SBL_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE SEAFLUX_SBL_ALLOC

SUBROUTINE SEAFLUX_SBL_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_SEAFLUX_SBL_N:SEAFLUX_SBL_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(SEAFLUX_SBL_MODEL)) DEALLOCATE(SEAFLUX_SBL_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_SEAFLUX_SBL_N:SEAFLUX_SBL_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE SEAFLUX_SBL_DEALLO

END MODULE MODD_SEAFLUX_SBL_n
