!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ####################
      MODULE MODD_SURF_ATM_SSO_n
!     ######################
!
!!****  *MODD_SURF_ATM_SSO - declaration of surface parameters related to orography
!!
!!    PURPOSE
!!    -------
!     Declaration of surface parameters
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	V. Masson    *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       01/2004
!
!*       0.   DECLARATIONS
!             ------------
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE SURF_ATM_SSO_t
!
!-----------------------------------------------------------------------------------------------------
!
! Type of roughness
!
 CHARACTER(LEN=4) :: CROUGH     ! type of orographic roughness
!                              ! 'NONE'
                               ! 'Z01D'
                               ! 'Z04D'
                               ! 'BE04'

!-----------------------------------------------------------------------------------------------------
!
! Subgrid orography parameters
!
  REAL, DIMENSION(:), POINTER :: XAOSIP,XAOSIM,XAOSJP,XAOSJM
! directional A/S quantities in 4 coordinate directions
! (IP: i index up;  IM: i index down;  JP: j index up;  JM: j index down)
! They are used in soil routines to compute effective roughness length
!
  REAL, DIMENSION(:), POINTER :: XHO2IP,XHO2IM,XHO2JP,XHO2JM
! directional h/2 quantities in 4 coordinate directions
! (IP: i index up;  IM: i index down;  JP: j index up;  JM: j index down)
! They are used in soil routines to compute effective roughness length
!
  REAL, DIMENSION(:), POINTER :: XZ0EFFIP,XZ0EFFIM,XZ0EFFJP,XZ0EFFJM
! directional total roughness lenghts in 4 coordinate directions
! (IP: i index up;  IM: i index down;  JP: j index up;  JM: j index down)
!
  REAL, DIMENSION(:), POINTER   :: XZ0EFFJPDIR    ! heading of J direction (deg from N clockwise)

  REAL, DIMENSION(:), POINTER   :: XZ0REL         ! relief roughness length                 (m)
!
  REAL, DIMENSION(:), POINTER   :: XSSO_SLOPE         ! slope of S.S.O.
  REAL, DIMENSION(:), POINTER   :: XSSO_ANIS          ! anisotropy of S.S.O.
  REAL, DIMENSION(:), POINTER   :: XSSO_DIR           ! direction of S.S.O. (deg from N clockwise) 
  REAL, DIMENSION(:), POINTER   :: XSSO_STDEV         ! S.S.O. standard deviation           (m)
!
!
  REAL, DIMENSION(:), POINTER   :: XAVG_ZS        ! averaged orography                      (m)
  REAL, DIMENSION(:), POINTER   :: XSIL_ZS        ! silhouette orography                    (m)
  REAL, DIMENSION(:), POINTER   :: XMAX_ZS        ! maximum subgrid orography               (m)
  REAL, DIMENSION(:), POINTER   :: XMIN_ZS        ! minimum subgrid orography               (m)
! Zo threshold
  REAL   :: XFRACZ0                                ! Z0=Min(Z0, Href/XFRACZ0)
  REAL   :: XCOEFBE                                ! Beljaars coefficient         
!-----------------------------------------------------------------------------------------------------
!
!


END TYPE SURF_ATM_SSO_t

TYPE(SURF_ATM_SSO_t), ALLOCATABLE, TARGET, SAVE :: SURF_ATM_SSO_MODEL(:)

 CHARACTER(LEN=4),   POINTER :: CROUGH=>NULL()
!$OMP THREADPRIVATE(CROUGH)
REAL, DIMENSION(:), POINTER :: XAOSIP=>NULL(),XAOSIM=>NULL(),XAOSJP=>NULL(),XAOSJM=>NULL()
!$OMP THREADPRIVATE(XAOSIP,XAOSIM,XAOSJP,XAOSJM)
REAL, DIMENSION(:), POINTER :: XHO2IP=>NULL(),XHO2IM=>NULL(),XHO2JP=>NULL(),XHO2JM=>NULL()
!$OMP THREADPRIVATE(XHO2IP,XHO2IM,XHO2JP,XHO2JM)
REAL, DIMENSION(:), POINTER :: XZ0EFFIP=>NULL(),XZ0EFFIM=>NULL(),XZ0EFFJP=>NULL(),XZ0EFFJM=>NULL()
!$OMP THREADPRIVATE(XZ0EFFIP,XZ0EFFIM,XZ0EFFJP,XZ0EFFJM)
REAL, DIMENSION(:), POINTER   :: XZ0EFFJPDIR=>NULL()
!$OMP THREADPRIVATE(XZ0EFFJPDIR)
REAL, DIMENSION(:), POINTER   :: XZ0REL=>NULL()
!$OMP THREADPRIVATE(XZ0REL)
REAL, DIMENSION(:), POINTER   :: XSSO_SLOPE=>NULL()
!$OMP THREADPRIVATE(XSSO_SLOPE)
REAL, DIMENSION(:), POINTER   :: XSSO_ANIS=>NULL()
!$OMP THREADPRIVATE(XSSO_ANIS)
REAL, DIMENSION(:), POINTER   :: XSSO_DIR=>NULL()
!$OMP THREADPRIVATE(XSSO_DIR)
REAL, DIMENSION(:), POINTER   :: XSSO_STDEV=>NULL()
!$OMP THREADPRIVATE(XSSO_STDEV)
REAL, DIMENSION(:), POINTER   :: XAVG_ZS=>NULL()
!$OMP THREADPRIVATE(XAVG_ZS)
REAL, DIMENSION(:), POINTER   :: XSIL_ZS=>NULL()
!$OMP THREADPRIVATE(XSIL_ZS)
REAL, DIMENSION(:), POINTER   :: XMAX_ZS=>NULL()
!$OMP THREADPRIVATE(XMAX_ZS)
REAL, DIMENSION(:), POINTER   :: XMIN_ZS=>NULL()
!$OMP THREADPRIVATE(XMIN_ZS)
REAL, POINTER   :: XFRACZ0=>NULL()
!$OMP THREADPRIVATE(XFRACZ0)
REAL, POINTER   :: XCOEFBE=>NULL()
!$OMP THREADPRIVATE(XCOEFBE)

CONTAINS

SUBROUTINE SURF_ATM_SSO_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
SURF_ATM_SSO_MODEL(KFROM)%XAOSIP=>XAOSIP
SURF_ATM_SSO_MODEL(KFROM)%XAOSIM=>XAOSIM
SURF_ATM_SSO_MODEL(KFROM)%XAOSJP=>XAOSJP
SURF_ATM_SSO_MODEL(KFROM)%XAOSJM=>XAOSJM
SURF_ATM_SSO_MODEL(KFROM)%XHO2IP=>XHO2IP
SURF_ATM_SSO_MODEL(KFROM)%XHO2IM=>XHO2IM
SURF_ATM_SSO_MODEL(KFROM)%XHO2JP=>XHO2JP
SURF_ATM_SSO_MODEL(KFROM)%XHO2JM=>XHO2JM
SURF_ATM_SSO_MODEL(KFROM)%XZ0EFFIP=>XZ0EFFIP
SURF_ATM_SSO_MODEL(KFROM)%XZ0EFFIM=>XZ0EFFIM
SURF_ATM_SSO_MODEL(KFROM)%XZ0EFFJP=>XZ0EFFJP
SURF_ATM_SSO_MODEL(KFROM)%XZ0EFFJM=>XZ0EFFJM
SURF_ATM_SSO_MODEL(KFROM)%XZ0EFFJPDIR=>XZ0EFFJPDIR
SURF_ATM_SSO_MODEL(KFROM)%XZ0REL=>XZ0REL
SURF_ATM_SSO_MODEL(KFROM)%XSSO_SLOPE=>XSSO_SLOPE
SURF_ATM_SSO_MODEL(KFROM)%XSSO_ANIS=>XSSO_ANIS
SURF_ATM_SSO_MODEL(KFROM)%XSSO_DIR=>XSSO_DIR
SURF_ATM_SSO_MODEL(KFROM)%XSSO_STDEV=>XSSO_STDEV
SURF_ATM_SSO_MODEL(KFROM)%XAVG_ZS=>XAVG_ZS
SURF_ATM_SSO_MODEL(KFROM)%XSIL_ZS=>XSIL_ZS
SURF_ATM_SSO_MODEL(KFROM)%XMAX_ZS=>XMAX_ZS
SURF_ATM_SSO_MODEL(KFROM)%XMIN_ZS=>XMIN_ZS
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_SURF_ATM_SSO_N:SURF_ATM_SSO_GOTO_MODEL',0,ZHOOK_HANDLE)
CROUGH=>SURF_ATM_SSO_MODEL(KTO)%CROUGH
XFRACZ0=>SURF_ATM_SSO_MODEL(KTO)%XFRACZ0
XCOEFBE=>SURF_ATM_SSO_MODEL(KTO)%XCOEFBE
XAOSIP=>SURF_ATM_SSO_MODEL(KTO)%XAOSIP
XAOSIM=>SURF_ATM_SSO_MODEL(KTO)%XAOSIM
XAOSJP=>SURF_ATM_SSO_MODEL(KTO)%XAOSJP
XAOSJM=>SURF_ATM_SSO_MODEL(KTO)%XAOSJM
XHO2IP=>SURF_ATM_SSO_MODEL(KTO)%XHO2IP
XHO2IM=>SURF_ATM_SSO_MODEL(KTO)%XHO2IM
XHO2JP=>SURF_ATM_SSO_MODEL(KTO)%XHO2JP
XHO2JM=>SURF_ATM_SSO_MODEL(KTO)%XHO2JM
XZ0EFFIP=>SURF_ATM_SSO_MODEL(KTO)%XZ0EFFIP
XZ0EFFIM=>SURF_ATM_SSO_MODEL(KTO)%XZ0EFFIM
XZ0EFFJP=>SURF_ATM_SSO_MODEL(KTO)%XZ0EFFJP
XZ0EFFJM=>SURF_ATM_SSO_MODEL(KTO)%XZ0EFFJM
XZ0EFFJPDIR=>SURF_ATM_SSO_MODEL(KTO)%XZ0EFFJPDIR
XZ0REL=>SURF_ATM_SSO_MODEL(KTO)%XZ0REL
XSSO_SLOPE=>SURF_ATM_SSO_MODEL(KTO)%XSSO_SLOPE
XSSO_ANIS=>SURF_ATM_SSO_MODEL(KTO)%XSSO_ANIS
XSSO_DIR=>SURF_ATM_SSO_MODEL(KTO)%XSSO_DIR
XSSO_STDEV=>SURF_ATM_SSO_MODEL(KTO)%XSSO_STDEV
XAVG_ZS=>SURF_ATM_SSO_MODEL(KTO)%XAVG_ZS
XSIL_ZS=>SURF_ATM_SSO_MODEL(KTO)%XSIL_ZS
XMAX_ZS=>SURF_ATM_SSO_MODEL(KTO)%XMAX_ZS
XMIN_ZS=>SURF_ATM_SSO_MODEL(KTO)%XMIN_ZS
IF (LHOOK) CALL DR_HOOK('MODD_SURF_ATM_SSO_N:SURF_ATM_SSO_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE SURF_ATM_SSO_GOTO_MODEL

SUBROUTINE SURF_ATM_SSO_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_SURF_ATM_SSO_N:SURF_ATM_SSO_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(SURF_ATM_SSO_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XAOSIP)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XAOSIM)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XAOSJP)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XAOSJM)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XHO2IP)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XHO2IM)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XHO2JP)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XHO2JM)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XZ0EFFIP)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XZ0EFFIM)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XZ0EFFJP)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XZ0EFFJM)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XZ0EFFJPDIR)  
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XZ0REL)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XSSO_SLOPE)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XSSO_ANIS)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XSSO_DIR)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XSSO_STDEV)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XAVG_ZS)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XSIL_ZS)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XMAX_ZS)
  NULLIFY(SURF_ATM_SSO_MODEL(J)%XMIN_ZS)
ENDDO
SURF_ATM_SSO_MODEL(:)%CROUGH=' '
SURF_ATM_SSO_MODEL(:)%XFRACZ0=2.
SURF_ATM_SSO_MODEL(:)%XCOEFBE=2.
IF (LHOOK) CALL DR_HOOK("MODD_SURF_ATM_SSO_N:SURF_ATM_SSO_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE SURF_ATM_SSO_ALLOC

SUBROUTINE SURF_ATM_SSO_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_SURF_ATM_SSO_N:SURF_ATM_SSO_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(SURF_ATM_SSO_MODEL)) DEALLOCATE(SURF_ATM_SSO_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_SURF_ATM_SSO_N:SURF_ATM_SSO_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE SURF_ATM_SSO_DEALLO

END MODULE MODD_SURF_ATM_SSO_n
